package cn.minbb.messagebarrage;

import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.TypedArray;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.support.v4.graphics.drawable.DrawableCompat;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.view.animation.Animation;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.bumptech.glide.Glide;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import static cn.minbb.messagebarrage.MessageBarrageTools.getScreenWidth;

public class MessageBarrageView extends RelativeLayout {

    private List<Integer> existMarginValues = new ArrayList<>();
    private int linesCount;

    private int validHeightSpace;

    private Random random = new Random(System.currentTimeMillis());
    private int maxBarrageSize;
    private int lineHeight;
    private int borderColor;
    private boolean random_color;
    private int DEFAULT_BARRAGESIZE = 10;
    private int DEFAULT_LINEHEIGHT = 24;
    private int DEFAULT_BORDERCOLOR = 0xff000000;
    private boolean DEFAULT_RANDOMCOLOR = false;

    private Context context;
    private View view;

    public MessageBarrageView(Context context) {
        this(context, null);
    }

    public MessageBarrageView(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public MessageBarrageView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.context = context;
        TypedArray typedArray = context.getTheme().obtainStyledAttributes(attrs, R.styleable.MessageBarrageView, 0, 0);
        try {
            maxBarrageSize = typedArray.getInt(R.styleable.MessageBarrageView_size, DEFAULT_BARRAGESIZE);
            lineHeight = typedArray.getDimensionPixelSize(R.styleable.MessageBarrageView_line_height, MessageBarrageTools.dp2px(context, DEFAULT_LINEHEIGHT));
            borderColor = typedArray.getColor(R.styleable.MessageBarrageView_border_color, DEFAULT_BORDERCOLOR);
            random_color = typedArray.getBoolean(R.styleable.MessageBarrageView_random_color, DEFAULT_RANDOMCOLOR);
        } finally {
            typedArray.recycle();
        }
    }

    public void showBarrage(MessageBarrage messageBarrage) {
        if (linesCount != 0 && getChildCount() >= linesCount)
            return;
        if (getChildCount() >= maxBarrageSize)
            return;
        // final TextView textView = tb.isShowBorder() ? new BorderTextView(getContext(), borderColor) : new TextView(getContext());
        view = LayoutInflater.from(context).inflate(R.layout.message_barrage_item, null);
        CircleImageView avatar = view.findViewById(R.id.avatar);
        TextView name = view.findViewById(R.id.name);
        TextView message = view.findViewById(R.id.message);

        Drawable drawable = view.getContext().getResources().getDrawable(R.drawable.shape_bg_round);
        view.setBackgroundDrawable(tintDrawable(drawable, messageBarrage.getBackgroundColor()));
        // view.setPadding(textLeftPadding, textTopPadding, textRightPadding, textBottomPadding);
        view.setPadding(8, 8, 8, 8);

        Glide.with(context).load(messageBarrage.getAvatar()).into(avatar);
        name.setText(messageBarrage.getName());
        message.setText(messageBarrage.getMessage());
        int color = random_color ? Color.rgb(random.nextInt(256), random.nextInt(256), random.nextInt(256)) : getResources().getColor(messageBarrage.getColor());
        name.setTextColor(color);
        message.setTextColor(color);

        int leftMargin = getRight() - getLeft() - getPaddingLeft();
        int verticalMargin = getRandomTopMargin();
        view.setTag(verticalMargin);
        LayoutParams params = new LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
        params.addRule(RelativeLayout.ALIGN_PARENT_TOP);
        params.topMargin = verticalMargin;
        view.setLayoutParams(params);
        Animation anim = AnimationHelper.createTranslateAnim(getContext(), leftMargin, -getScreenWidth(getContext()));
        anim.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {
            }

            @Override
            public void onAnimationEnd(Animation animation) {
                removeView(view);
                // int verticalMargin = (int) view.getTag();
                if (existMarginValues.size() > 2) {
                    existMarginValues.remove(existMarginValues.get(0));
                }
            }

            @Override
            public void onAnimationRepeat(Animation animation) {
            }
        });
        view.startAnimation(anim);
        addView(view);
    }

    private int getRandomTopMargin() {
        if (validHeightSpace == 0) {
            validHeightSpace = getBottom() - getTop() - getPaddingTop() - getPaddingBottom();
        }
        if (linesCount == 0) {
            linesCount = validHeightSpace / lineHeight;
            if (linesCount == 0) {
                throw new RuntimeException("Not enough space to show text.");
            }
        }
        while (true) {
            int randomIndex = (int) (Math.random() * linesCount);
            int marginValue = randomIndex * (validHeightSpace / linesCount);
            if (marginValue < validHeightSpace - lineHeight) {
                if (!existMarginValues.contains(marginValue)) {
                    existMarginValues.add(marginValue);
                    return marginValue;
                }
            }
        }
    }

    public void destroy() {
    }

    private Drawable tintDrawable(Drawable drawable, int color) {
        ColorStateList colors = ColorStateList.valueOf(color);
        Drawable wrappedDrawable = DrawableCompat.wrap(drawable);
        DrawableCompat.setTintList(wrappedDrawable, colors);
        return wrappedDrawable;
    }
}
