package cn.minbb.evaluation;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.annotation.NonNull;
import android.support.design.widget.BottomNavigationView;
import android.support.design.widget.Snackbar;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.ViewPager;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.android.volley.Request;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.StringRequest;
import com.bumptech.glide.Glide;
import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.adapter.MainActivityPagerAdapter;
import cn.minbb.evaluation.adapter.MainTabThreeListAdapter;
import cn.minbb.evaluation.adapter.WaterfallStaggeredAdapter;
import cn.minbb.evaluation.views.CircleImageView;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.PermissionManager;
import cn.minbb.evaluation.ctrl.UpdateManager;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.dao.ClassificationDao;
import cn.minbb.evaluation.dao.DatabaseManager;
import cn.minbb.evaluation.entity.Classification;
import cn.minbb.evaluation.entity.Message;
import cn.minbb.evaluation.entity.User;
import cn.minbb.evaluation.entity.Version;
import cn.minbb.evaluation.fragment.AvatarFragment;
import cn.minbb.evaluation.receiver.FinishReceiver;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;
import cn.minbb.evaluation.activities.AboutActivity;
import cn.minbb.evaluation.activities.EvaluationActivity;
import cn.minbb.evaluation.activities.FeedbackActivity;
import cn.minbb.evaluation.activities.HistoryPaperActivity;
import cn.minbb.evaluation.activities.PublicChatActivity;
import cn.minbb.evaluation.activities.SettingsActivity;
import cn.minbb.evaluation.activities.UserInfoActivity;
import cn.minbb.messagebarrage.MessageBarrage;
import cn.minbb.messagebarrage.MessageBarrageView;
import io.reactivex.functions.Consumer;
import okhttp3.WebSocket;
import ua.naiksoftware.stomp.Stomp;
import ua.naiksoftware.stomp.client.StompClient;
import ua.naiksoftware.stomp.client.StompMessage;

public class MainActivity extends AppCompatActivity implements ViewPager.OnPageChangeListener,
        BottomNavigationView.OnNavigationItemSelectedListener, AdapterView.OnItemClickListener,
        SwipeRefreshLayout.OnRefreshListener, WaterfallStaggeredAdapter.OnItemClickListener {

    @BindView(R.id.viewPager)
    ViewPager viewPager;
    @BindView(R.id.navigation)
    BottomNavigationView navigation;

    private SwipeRefreshLayout mainTab_1_refreshLayout;
    private RecyclerView mainTab_1_refreshRecycle;

    private List<View> views = new ArrayList<>(3);
    private MessageBarrageView publicChatMessageBarrageView;
    private CircleImageView avatar;
    private TextView nameText;
    private ListView mainTab_3_list;

    private FinishReceiver finishReceiver;

    private ClassificationDao classificationDao = ClassificationDao.getClassificationDao();

    private User user = null;

    private StompClient stompClient;
    private Timer reconnectStomp;
    private boolean autoReconnect = true;        // 自动重连

    private SharedPreferences preferences;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 启用通过资源加载矢量图 - Android 5.0 以前版本必须使用
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);
        preferences = PreferenceManager.getDefaultSharedPreferences(this);

        initView();
        setListener();
        loadLocalData();
        loadServerData();
    }

    @Override
    protected void onPause() {
        super.onPause();
        closeSocket();
    }

    @Override
    protected void onResume() {
        super.onResume();
        openSocket();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        // 注销动态注册的广播
        unregisterReceiver(finishReceiver);
        // 取消所有网络请求
        App.getRequestQueue().cancelAll("versionRequest");
        App.getRequestQueue().cancelAll("classificationRequest");
        closeSocket();
        // 释放弹幕视图
        publicChatMessageBarrageView.destroy();
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, MainActivity.class));
    }

    @SuppressLint({"InflateParams"})
    private void initView() {
        LayoutInflater layoutInflater = LayoutInflater.from(this);
        View mainTab_1, mainTab_2, mainTab_3;
        mainTab_1 = layoutInflater.inflate(R.layout.main_tab_1, null);
        mainTab_2 = layoutInflater.inflate(R.layout.main_tab_2, null);
        mainTab_3 = layoutInflater.inflate(R.layout.main_tab_3, null);
        views.add(mainTab_1);
        views.add(mainTab_2);
        views.add(mainTab_3);
        viewPager.setAdapter(new MainActivityPagerAdapter(views));

        mainTab_1_refreshLayout = mainTab_1.findViewById(R.id.mainTab_1_refreshLayout);
        mainTab_1_refreshLayout.setColorSchemeResources(
                android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);
        mainTab_1_refreshRecycle = mainTab_1.findViewById(R.id.mainTab_1_refreshRecycle);
        publicChatMessageBarrageView = mainTab_2.findViewById(R.id.publicChatMessageBarrageView);
        avatar = mainTab_3.findViewById(R.id.avatar);
        nameText = mainTab_3.findViewById(R.id.nameText);
        mainTab_3_list = mainTab_3.findViewById(R.id.mainTab_3_list);

        List<MainTabThreeListAdapter.ItemBean> beanList_3 = new ArrayList<>();
        beanList_3.add(new MainTabThreeListAdapter.ItemBean("{fa-cog spin}   设置", "设置相关"));
        beanList_3.add(new MainTabThreeListAdapter.ItemBean("{fa-info-circle spin}   关于", "IEC创新创业中心"));
        beanList_3.add(new MainTabThreeListAdapter.ItemBean("{fa-envelope spin 90%}   反馈", "向我们提宝贵意见"));
        mainTab_3_list.setAdapter(new MainTabThreeListAdapter(this, beanList_3));

        // 自动检查更新
        JsonObjectRequest versionRequest = new JsonObjectRequest(Request.Method.POST,
                Constant.SERVER_HOST + "version",
                null,
                jsonObject -> {
                    try {
                        JSONObject json = JSONObject.parseObject(jsonObject.toString());
                        switch (json.getInteger(ResponseMap.RESULT)) {
                            case Result.SUCCESS:
                                new UpdateManager(this, JSONObject.parseObject(json.getString(ResponseMap.DATA), Version.class)).checkUpdate();
                                break;
                            case Result.FAILED:
                                Snackbar.make(getWindow().getDecorView(), "自动检查更新失败", Snackbar.LENGTH_SHORT).show();
                                break;
                            default:
                        }
                    } catch (Exception e) {
                        Snackbar.make(getWindow().getDecorView(), "请检查网络连接状态", Snackbar.LENGTH_SHORT).show();
                    }
                }, volleyError -> {
            Snackbar.make(getWindow().getDecorView(), "自动检查更新失败\n请检查网络连接状态", Snackbar.LENGTH_SHORT).show();
        });
        versionRequest.setTag("versionRequest");
        App.getRequestQueue().add(versionRequest);

        // 动态申请权限
        new PermissionManager(this).addPermission(null).checkAndRequestPermissions();
    }

    private void setListener() {
        // 动态注册一个结束当前活动的广播，Action 为 cn.minbb.innovationabilityevaluationsystem.MainActivity.Finish
        finishReceiver = new FinishReceiver(this);
        registerReceiver(finishReceiver, new IntentFilter("cn.minbb.innovationabilityevaluationsystem.MainActivity.Finish"));
        viewPager.addOnPageChangeListener(this);
        navigation.setOnNavigationItemSelectedListener(this);
        mainTab_1_refreshLayout.setOnRefreshListener(this);
        mainTab_3_list.setOnItemClickListener(this);
    }

    private void loadLocalData() {
        List<Classification> classificationList = classificationDao.findAll();
        List<WaterfallStaggeredAdapter.ItemBean> beanList_1 = new ArrayList<>();
        for (Classification classification : classificationList) {
            beanList_1.add(new WaterfallStaggeredAdapter.ItemBean(classification.getId(), classification.getName(), classification.getIntroduction()));
        }
        WaterfallStaggeredAdapter waterfallStaggeredAdapter = new WaterfallStaggeredAdapter(this, beanList_1);
        waterfallStaggeredAdapter.setOnItemClickListener(this);
        // LayoutAnimationController animation = AnimationUtils.loadLayoutAnimation(this, R.anim.layout_animation_fall_down);
        // mainTab_1_refreshRecycle.setLayoutAnimation(animation);
        mainTab_1_refreshRecycle.setAdapter(waterfallStaggeredAdapter);
        // new Stagger 布局管理器，布局管理器所需参数:1.规定显示几列 2.item 排列方向
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL);
        // 通过布局管理器控制条目排列的顺序  true:反向显示 false:正向显示
        staggeredGridLayoutManager.setReverseLayout(false);
        // 设置布局管理器，参数 StaggeredGridLayoutManager，可以是 RecyclerView 实现和 StaggeredGridView 一样的效果
        mainTab_1_refreshRecycle.setLayoutManager(staggeredGridLayoutManager);
        // 数据变化时动画
        // LayoutAnimationController controller = AnimationUtils.loadLayoutAnimation(this, R.anim.layout_animation_fall_down);
        // mainTab_1_refreshRecycle.setLayoutAnimation(controller);
        // mainTab_1_refreshRecycle.getAdapter().notifyDataSetChanged();
        // mainTab_1_refreshRecycle.scheduleLayoutAnimation();

        user = UserManager.getCurrentUser();
        if (user != null) {
            if (TextUtils.isEmpty(user.getAvatar())) {
                avatar.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.ic_person_white_24dp));
            } else {
                Glide.with(this)
                        .load(user.getAvatar())
                        .placeholder(R.drawable.ic_person_white_24dp)
                        .error(R.drawable.ic_person_white_24dp)
                        .into(avatar);
            }
            if (TextUtils.isEmpty(user.getName())) {
                nameText.setText("快去设置一个吧！");
            } else {
                nameText.setText(user.getName());
            }
        }
    }

    private void loadServerData() {
        mainTab_1_refreshLayout.setRefreshing(true);

        StringRequest classificationRequest = new StringRequest(Request.Method.POST,
                Constant.SERVER_HOST + "classification",
                s -> {
                    mainTab_1_refreshLayout.setRefreshing(false);
                    try {
                        JSONObject jsonObject = JSONObject.parseObject(s);
                        switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                            case Result.SUCCESS:
                                classificationDao.deleteAll();
                                DatabaseManager.resetClassificationId();
                                List<Classification> classificationList = JSONObject.parseArray(jsonObject.getString(ResponseMap.DATA), Classification.class);
                                classificationDao.saveAll(classificationList);
                                loadLocalData();
                                break;
                            case Result.FAILED:
                                Snackbar.make(getWindow().getDecorView(), "获取数据失败\n" + jsonObject.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                                break;
                            default:
                        }
                    } catch (Exception e) {
                        Snackbar.make(getWindow().getDecorView(), "请检查网络连接状态", Snackbar.LENGTH_SHORT).show();
                    }
                },
                volleyError -> {
                    mainTab_1_refreshLayout.setRefreshing(false);
                    Snackbar.make(getWindow().getDecorView(), "获取数据失败，请重试！\n（或许大概是服务器不在线吧！）", Snackbar.LENGTH_LONG).show();
                });
        classificationRequest.setTag("classificationRequest");
        App.getRequestQueue().add(classificationRequest);

        openSocket();
    }

    /**
     * 初始化 WebSocket 连接
     */
    @SuppressLint("CheckResult")
    private void openSocket() {
        if (stompClient == null) {
            int id = 1;
            stompClient = Stomp.over(WebSocket.class, Constant.SERVER_WS_HOST);
            stompClient.connect();
            stompClient.lifecycle().subscribe(lifecycleEvent -> {
                switch (lifecycleEvent.getType()) {
                    case OPENED:
                        System.out.println("连接已开启");
                        break;
                    case ERROR:
                        System.out.println("连接出错 = " + lifecycleEvent.getException());
                        reconnectSocket();
                        break;
                    case CLOSED:
                        System.out.println("连接已关闭，自动连接 = " + autoReconnect);
                        if (autoReconnect) {
                            reconnectSocket();
                        }
                        autoReconnect = true;
                        break;
                    default:
                }
            });
            stompClient.topic("/queue/online/" + id).subscribe(stompMessage -> {
                runOnUiThread(() -> Toast.makeText(MainActivity.this, stompMessage.getPayload(), Toast.LENGTH_SHORT).show());
            });
            if (preferences.getBoolean("enable_chat_all", true)) {
                // 订阅全局公共聊天室消息
                stompClient.topic("/topic/chat/all").subscribe(new Consumer<StompMessage>() {
                    @Override
                    public void accept(StompMessage stompMessage) {
                        runOnUiThread(() -> {
                            // 当 MainActivity 在前台且在消息页面，显示弹幕消息流
                            if (App.activityIsForeground(MainActivity.this, "cn.minbb.evaluationsystem.MainActivity") && viewPager.getCurrentItem() == 1) {
                                Message message = JSONObject.parseObject(stompMessage.getPayload(), Message.class);
                                User userFrom = message.getUserFrom();
                                String avatar = "http://www.th7.cn/d/file/p/2016/08/11/44beef795460f5ef97de7950eb9006cf.jpg";
                                if (userFrom != null) {
                                    publicChatMessageBarrageView.showBarrage(new MessageBarrage(avatar, userFrom.getName() + ": ", message.getContent()));
                                } else {
                                    publicChatMessageBarrageView.showBarrage(new MessageBarrage(avatar, "匿名: ", message.getContent()));
                                }
                            }
                        });
                    }
                });
            }
        }
    }

    /**
     * 重新连接 WebSocket
     */
    private void reconnectSocket() {
        if (stompClient != null) {
            stompClient.disconnect();
            stompClient = null;
        }
        reconnectStomp = new Timer();
        reconnectStomp.schedule(new TimerTask() {
            @Override
            public void run() {
                System.out.println("自动重连");
                openSocket();
                reconnectStomp.cancel();
            }
        }, 5 * 1000);
    }

    /**
     * 断开 WebSocket 连接
     */
    private void closeSocket() {
        autoReconnect = false;
        // 取消自动重连并断开 WebSocket 长连接
        if (reconnectStomp != null) {
            reconnectStomp.cancel();
        }
        if (stompClient != null && stompClient.isConnected()) {
            stompClient.disconnect();
        }
    }

    public void onViewClicked(View v) {
        switch (v.getId()) {
            case R.id.publicChatView:
                PublicChatActivity.start(this);
                break;
            case R.id.mainTab_3_1:
                UserInfoActivity.start(this);
                break;
            case R.id.mainTab_3_2:
                HistoryPaperActivity.start(this);
                break;
            case R.id.mainTab_3_3:
                break;
            case R.id.avatar:
            case R.id.nameText:
                String avatarURL = user.getAvatar();
                if (!TextUtils.isEmpty(avatarURL)) {
                    FragmentManager fragmentManager = getSupportFragmentManager();
                    FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
                    // 设置 Fragment 动画
                    fragmentTransaction.setCustomAnimations(R.anim.avatar_in, R.anim.avatar_out, R.anim.avatar_in, R.anim.avatar_out);
                    fragmentTransaction.addToBackStack(null).add(R.id.main, new AvatarFragment(), "").commit();
                } else {
                    Snackbar.make(getWindow().getDecorView(), "您还没有设置头像", Snackbar.LENGTH_LONG)
                            .setAction("现在去设置", new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                    UserInfoActivity.start(MainActivity.this);
                                }
                            }).show();
                }
                break;
            default:
        }
    }

    // WaterfallStaggeredAdapter 点击事件
    @Override
    public void onItemClick(View view, int position) {
        Classification classification = classificationDao.findByE_Id((Long) view.getTag());
        EvaluationActivity.start(this, JSONObject.toJSONString(classification));
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        switch (parent.getId()) {
            case R.id.mainTab_3_list:
                switch (position) {
                    case 0:
                        SettingsActivity.start(this);
                        break;
                    case 1:
                        AboutActivity.start(this);
                        break;
                    case 2:
                        FeedbackActivity.start(this);
                        break;
                    default:
                }
                break;
            default:
        }
    }

    @Override
    public void onRefresh() {
        loadServerData();
    }

    @Override
    public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
    }

    @Override
    public void onPageSelected(int position) {
        switch (position) {
            case 0:
                navigation.setSelectedItemId(R.id.navigation_1);
                break;
            case 1:
                navigation.setSelectedItemId(R.id.navigation_2);
                break;
            case 2:
                navigation.setSelectedItemId(R.id.navigation_3);
                break;
            default:
        }
    }

    @Override
    public void onPageScrollStateChanged(int state) {
    }

    /**
     * 底部导航选择监听器
     *
     * @param item 导航菜单项
     * @return
     */
    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem item) {
        switch (item.getItemId()) {
            case R.id.navigation_1:
                viewPager.setCurrentItem(0);
                return true;
            case R.id.navigation_2:
                viewPager.setCurrentItem(1);
                return true;
            case R.id.navigation_3:
                viewPager.setCurrentItem(2);
                return true;
        }
        return false;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        switch (requestCode) {
            case Constant.PERMISSIONS_REQUEST:
                // If request is cancelled, the result arrays are empty.
                int length = grantResults.length;
                // 标记位：如果需要重新授予权限，true；反之，false。
                boolean reRequest = false;
                for (int i = 0; i < length; i++) {
                    if (grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                        System.out.println("权限授予成功:" + permissions[i]);
                    } else {
                        System.out.println("权限授予失败:" + permissions[i]);
                        reRequest = true;
                    }
                }
                if (reRequest) {
                    // 弹出对话框，提示用户重新授予权限
                    new AlertDialog.Builder(this)
                            .setIcon(new IconDrawable(this, FontAwesomeIcons.fa_warning)
                                    .colorRes(android.R.color.black)
                                    .actionBarSize())
                            .setTitle("请求授权")
                            .setMessage("请授予必要的 SD 卡读写权限，\n否则程序无法正常运行。\n\n点击确定，重新授予权限。\n点击取消，部分功能将受到限制。")
                            .setPositiveButton(getString(R.string.operation_confirm), new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                    // 弹出手动修改权限的系统界面
                                    Intent localIntent = new Intent();
                                    localIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                                    if (Build.VERSION.SDK_INT >= 9) {
                                        localIntent.setAction("android.settings.APPLICATION_DETAILS_SETTINGS");
                                        localIntent.setData(Uri.fromParts("package", App.getApp().getPackageName(), null));
                                    } else if (Build.VERSION.SDK_INT <= 8) {
                                        localIntent.setAction(Intent.ACTION_VIEW);
                                        localIntent.setClassName("com.android.settings", "com.android.settings.InstalledAppDetails");
                                        localIntent.putExtra("com.android.settings.ApplicationPkgName", App.getApp().getPackageName());
                                    }
                                    startActivity(localIntent);
                                }
                            })
                            .setNegativeButton(getString(R.string.operation_cancel), new DialogInterface.OnClickListener() {
                                @Override
                                public void onClick(DialogInterface dialog, int which) {
                                }
                            })
                            .setCancelable(false).create().show();
                }
                break;
            default:
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("退出？")
                    .setMessage("确定要退出吗？")
                    .setNegativeButton("取消", (dialog, which) -> {
                    })
                    .setPositiveButton("退出", (dialog, which) -> {
                        MainActivity.this.finish();
                    })
                    .create()
                    .show();
        }
        return super.onKeyDown(keyCode, event);
    }
}
