package cn.minbb.evaluation.activities;

import android.animation.Animator;
import android.animation.ObjectAnimator;
import android.animation.ValueAnimator;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.LinearInterpolator;
import android.widget.ImageView;
import android.widget.SeekBar;
import android.widget.TextView;

import java.util.Timer;
import java.util.TimerTask;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.views.CreditsRollView;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.utils.AnimationUtil;
import cn.minbb.evaluation.utils.ColorUtil;
import cn.minbb.heartlayout.HeartLayout;


public class AboutActivity extends AppCompatActivity implements SeekBar.OnSeekBarChangeListener {

    @BindView(R.id.appIcon)
    ImageView appIcon;
    @BindView(R.id.appName)
    TextView appName;
    @BindView(R.id.appTalk)
    CreditsRollView appTalk;
    @BindView(R.id.seekBar)
    SeekBar seekBar;
    @BindView(R.id.heartLayout)
    HeartLayout heartLayout;

    private boolean scrolling;
    private ValueAnimator scrollAnimator;
    private Timer timer = new Timer();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_about);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        timer.cancel();
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, AboutActivity.class));
    }

    private void initView() {
        App.setupActionBar(this);
        startScrollAnimation();
        timer.scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                heartLayout.post(new Runnable() {
                    @Override
                    public void run() {
                        heartLayout.addHeart(ColorUtil.getRandomColor());
                    }
                });
            }
        }, 500, 200);
    }

    private void setListener() {
        appIcon.setOnClickListener(this::onViewClicked);
        appName.setOnClickListener(this::onViewClicked);
        appTalk.setOnClickListener(this::onViewClicked);
        seekBar.setOnSeekBarChangeListener(this);
    }

    private void loadData() {
    }

    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.appIcon:
                AnimationUtil.setShakeAnimation(appIcon);
                break;
            case R.id.appName:
                AnimationUtil.setRotateShakeAnimation(appName);
                break;
            case R.id.appTalk:
                if (!scrolling) {
                    startScrollAnimation();
                } else {
                    stopScrollAnimation();
                }
                break;
            default:
        }
    }

    private void startScrollAnimation() {
        scrolling = true;
        scrollAnimator = ObjectAnimator.ofInt(seekBar, "progress", seekBar.getProgress(), seekBar.getMax());
        scrollAnimator.setDuration((long) (10000 * (1 - (float) seekBar.getProgress() / seekBar.getMax())));
        scrollAnimator.setInterpolator(new LinearInterpolator());
        scrollAnimator.addListener(new Animator.AnimatorListener() {
            @Override
            public void onAnimationStart(Animator animation) {
            }

            @Override
            public void onAnimationEnd(Animator animation) {
                scrolling = false;
            }

            @Override
            public void onAnimationCancel(Animator animation) {
            }

            @Override
            public void onAnimationRepeat(Animator animation) {
            }
        });
        scrollAnimator.start();
    }

    private void stopScrollAnimation() {
        if (scrollAnimator != null) {
            scrollAnimator.cancel();
            scrollAnimator = null;
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                AboutActivity.this.finish();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
        appTalk.setScrollPosition(progress / 10000f); // We have increments of 1/10000 %
    }

    @Override
    public void onStartTrackingTouch(SeekBar seekBar) {
        if (scrolling) {
            stopScrollAnimation();
        }
    }

    @Override
    public void onStopTrackingTouch(SeekBar seekBar) {
    }
}
