package cn.minbb.evaluation.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.alibaba.fastjson.JSONObject;

import java.io.IOException;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.Classification;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Response;

public class EvaluationActivity extends AppCompatActivity implements View.OnClickListener {

    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.fab)
    FloatingActionButton fab;
    @BindView(R.id.introduction)
    TextView introduction;

    private ProgressDialog dialog;

    private Classification classification;

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_evaluation);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
        }
    }

    public static void start(Context context, String classificationJSON) {
        Intent intent = new Intent(context, EvaluationActivity.class);
        intent.putExtra("classificationJSON", classificationJSON);
        context.startActivity(intent);
    }

    private void initView() {
        Intent intent = getIntent();
        classification = JSONObject.parseObject(intent.getStringExtra("classificationJSON"), Classification.class);
        toolbar.setNavigationIcon(R.drawable.ic_arrow_back_white_24dp);
        toolbar.setTitle(classification.getName());
        setSupportActionBar(toolbar);
        introduction.setText(classification.getIntroduction());
    }

    private void setListener() {
        fab.setOnClickListener(this);
    }

    private void loadData() {
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("抽题");
            dialog.setMessage("随机抽取试题中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                EvaluationActivity.this.finish();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.fab:
                AlertDialog.Builder builder = new AlertDialog.Builder(EvaluationActivity.this);
                builder.setTitle(R.string.i_tell_you)
                        .setMessage(classification.getRemark())
                        .setPositiveButton(getString(R.string.action_ok), (dialog, which) -> {
                            Snackbar.make(getWindow().getDecorView(), "祝您获得一个好成绩！", Snackbar.LENGTH_SHORT)
                                    .setAction(getString(R.string.action_ok), null).show();
                        })
                        .create().show();
                break;
            case R.id.startEvaluation:
                showProgress(true);
                OkHttpClient client = new OkHttpClient();
                FormBody.Builder formBodyBuilder = new FormBody.Builder()
                        .add("id", String.valueOf(classification.getE_id()))
                        .add("user", JSONObject.toJSONString(UserManager.getCurrentUser()));
                okhttp3.Request request = new okhttp3.Request.Builder()
                        .url(Constant.SERVER_HOST + "question")
                        .post(formBodyBuilder.build())
                        .build();
                client.newCall(request).enqueue(new Callback() {
                    @Override
                    public void onFailure(Call call, IOException e) {
                        EvaluationActivity.this.runOnUiThread(() -> {
                            showProgress(false);
                            Snackbar.make(toolbar.getRootView(), "获取数据失败", Snackbar.LENGTH_SHORT).show();
                        });
                    }

                    @Override
                    public void onResponse(Call call, Response response) throws IOException {
                        JSONObject jsonObject = JSONObject.parseObject(response.body().string());
                        EvaluationActivity.this.runOnUiThread(() -> {
                            showProgress(false);
                            switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                                case Result.SUCCESS:
                                    QuestionActivity.start(EvaluationActivity.this, jsonObject.getString(ResponseMap.DATA));
                                    EvaluationActivity.this.finish();
                                    break;
                                case Result.FAILED:
                                    Snackbar.make(getWindow().getDecorView(), "抽题失败！\n" + jsonObject.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                                    break;
                                default:
                            }
                        });
                    }
                });
                break;
            default:
        }
    }
}
