package cn.minbb.evaluation.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.IdRes;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.util.SparseBooleanArray;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.google.android.flexbox.FlexboxLayout;
import com.joanzapata.iconify.widget.IconTextView;

import java.io.IOException;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.views.ClearTextInputEditText;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.Feedback;
import cn.minbb.evaluation.utils.AnimationUtil;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;

public class FeedbackActivity extends AppCompatActivity implements RadioGroup.OnCheckedChangeListener, CompoundButton.OnCheckedChangeListener, TextWatcher {

    @BindView(R.id.typeTip)
    IconTextView typeTip;
    @BindView(R.id.type_1)
    RadioButton type1;
    @BindView(R.id.type_2)
    RadioButton type2;
    @BindView(R.id.type_3)
    RadioButton type3;
    @BindView(R.id.feedbackType)
    RadioGroup feedbackType;
    @BindView(R.id.problemTip)
    IconTextView problemTip;
    @BindView(R.id.problem_1)
    CheckBox problem1;
    @BindView(R.id.problem_2)
    CheckBox problem2;
    @BindView(R.id.problem_3)
    CheckBox problem3;
    @BindView(R.id.problem_4)
    CheckBox problem4;
    @BindView(R.id.problem_5)
    CheckBox problem5;
    @BindView(R.id.problem_6)
    CheckBox problem6;
    @BindView(R.id.problem_7)
    CheckBox problem7;
    @BindView(R.id.feedbackProblem)
    FlexboxLayout feedbackProblem;
    @BindView(R.id.problemContent)
    ClearTextInputEditText problemContent;
    @BindView(R.id.problemContentLimit)
    TextView problemContentLimit;
    @BindView(R.id.nameTip)
    IconTextView nameTip;
    @BindView(R.id.nameText)
    EditText nameText;
    @BindView(R.id.contactTip)
    IconTextView contactTip;
    @BindView(R.id.contactText)
    EditText contactText;
    @BindView(R.id.qqToDeveloper)
    IconTextView qqToDeveloper;
    @BindView(R.id.phoneToDeveloper)
    IconTextView phoneToDeveloper;
    @BindView(R.id.mailToDeveloper)
    IconTextView mailToDeveloper;

    private ProgressDialog dialog;

    private Feedback feedback = new Feedback();

    private SparseBooleanArray problems;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_feedback);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
        }
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, FeedbackActivity.class));
    }

    private void initView() {
        App.setupActionBar(this);
    }

    private void setListener() {
        feedbackType.setOnCheckedChangeListener(this);
        problem1.setOnCheckedChangeListener(this);
        problem2.setOnCheckedChangeListener(this);
        problem3.setOnCheckedChangeListener(this);
        problem4.setOnCheckedChangeListener(this);
        problem5.setOnCheckedChangeListener(this);
        problem6.setOnCheckedChangeListener(this);
        problem7.setOnCheckedChangeListener(this);
        problemContent.addTextChangedListener(this);
    }

    private void loadData() {
        problems = new SparseBooleanArray(7);
        for (int i = 0; i < 7; i++) {
            problems.put(i, false);
        }
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("提交");
            dialog.setMessage("努力提交数据中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    private void toFeedback() {
        // 取值
        StringBuilder feedbackTitle = new StringBuilder();
        for (int i = 0; i < problems.size(); i++) {
            if (problems.valueAt(i)) {
                switch (i) {
                    case 0:
                        feedbackTitle.append(getString(R.string.feedback_problem_1));
                        break;
                    case 1:
                        feedbackTitle.append(getString(R.string.feedback_problem_2));
                        break;
                    case 2:
                        feedbackTitle.append(getString(R.string.feedback_problem_3));
                        break;
                    case 3:
                        feedbackTitle.append(getString(R.string.feedback_problem_4));
                        break;
                    case 4:
                        feedbackTitle.append(getString(R.string.feedback_problem_5));
                        break;
                    case 5:
                        feedbackTitle.append(getString(R.string.feedback_problem_6));
                        break;
                    case 6:
                        feedbackTitle.append(getString(R.string.feedback_problem_7));
                        break;
                    default:
                        feedbackTitle.append("未知");
                }
                feedbackTitle.append("&");
            }
        }
        if (feedbackTitle.length() > 1) {
            // 去除最后一个字符
            feedbackTitle.deleteCharAt(feedbackTitle.length() - 1);
        }
        feedback.setTitle(feedbackTitle.toString());
        feedback.setName(nameText.getText().toString());
        feedback.setContact(contactText.getText().toString());

        // 检查表单完整性
        boolean cancel = false;
        View focusView = null;
        if (TextUtils.isEmpty(feedback.getType())) {
            cancel = true;
            focusView = typeTip;
        } else if (TextUtils.isEmpty(feedback.getTitle())) {
            cancel = true;
            focusView = feedbackProblem;
        } else if (TextUtils.isEmpty(feedback.getContent())) {
            cancel = true;
            focusView = problemTip;
        } else if (TextUtils.isEmpty(feedback.getName())) {
            cancel = true;
            focusView = nameTip;
        } else if (TextUtils.isEmpty(feedback.getContact())) {
            cancel = true;
            focusView = contactTip;
        }

        if (cancel) {
            AnimationUtil.setShakeAnimation(focusView);
        } else {
            showProgress(true);

            OkHttpClient client = new OkHttpClient();
            FormBody formBody = new FormBody.Builder()
                    .add("feedback", JSONObject.toJSONString(feedback))
                    .add("user", JSONObject.toJSONString(UserManager.getCurrentUser()))
                    .build();
            okhttp3.Request request = new okhttp3.Request.Builder()
                    .url(Constant.SERVER_HOST + "feedback")
                    .post(formBody)
                    .build();
            client.newCall(request).enqueue(new Callback() {
                @Override
                public void onFailure(Call call, IOException e) {
                    FeedbackActivity.this.runOnUiThread(() -> {
                        showProgress(false);
                        Toast.makeText(FeedbackActivity.this, "抱歉，提交信息失败，请重试！", Toast.LENGTH_SHORT).show();
                    });
                }

                @Override
                public void onResponse(Call call, okhttp3.Response response) throws IOException {
                    JSONObject jsonObject = JSONObject.parseObject(response.body().string());
                    FeedbackActivity.this.runOnUiThread(() -> {
                        showProgress(false);
                        FeedbackActivity.this.finish();
                        View view = getLayoutInflater().inflate(R.layout.feedback_toast, null);
                        ((TextView) view.findViewById(R.id.text)).setText(jsonObject.getString(ResponseMap.SUMMARY));
                        Toast toast = new Toast(FeedbackActivity.this);
                        // toast.setGravity(Gravity.BOTTOM, 0, 0);
                        toast.setDuration(Toast.LENGTH_SHORT);
                        toast.setView(view);
                        toast.show();
                        switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                            case Result.SUCCESS:
                                break;
                            case Result.FAILED:
                                break;
                        }
                    });
                }
            });
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 0, 0, getString(R.string.operation_submit)).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                FeedbackActivity.this.finish();
                break;
            case 0:
                toFeedback();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @OnClick({R.id.qqToDeveloper, R.id.phoneToDeveloper, R.id.mailToDeveloper})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.qqToDeveloper:
                try {
                    startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("mqqwpa://im/chat?chat_type=wpa&uin=1342875607")));
                } catch (Exception e) {
                    Toast.makeText(FeedbackActivity.this, "未检测到 QQ 程序", Toast.LENGTH_LONG).show();
                }
                break;
            case R.id.phoneToDeveloper:
                Intent intentPhone = new Intent(Intent.ACTION_DIAL, Uri.parse("tel:18838936922"));
                intentPhone.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                startActivity(intentPhone);
                break;
            case R.id.mailToDeveloper:
                Intent intentMail = new Intent(Intent.ACTION_SEND);
                // i.setType("text/plain");             // 模拟器
                intentMail.setType("message/rfc822");   // 真机
                intentMail.putExtra(Intent.EXTRA_EMAIL, new String[]{"1342875607@qq.com"});     // 收件人
                intentMail.putExtra(Intent.EXTRA_CC, new String[]{"yumin@minbb.cn"});           // 抄送者
                intentMail.putExtra(Intent.EXTRA_BCC, new String[]{"1342875607@qq.com"});       // 密送者
                intentMail.putExtra(Intent.EXTRA_SUBJECT, getString(R.string.app_name) + " APP 反馈与建议");
                intentMail.putExtra(Intent.EXTRA_TEXT, "我们很希望能得到您的宝贵建议！！！");
                startActivity(Intent.createChooser(intentMail, "请选择邮件应用"));
                break;
            default:
        }
    }

    @Override
    public void onCheckedChanged(RadioGroup group, @IdRes int checkedId) {
        if (group.getId() == R.id.feedbackType) {
            switch (checkedId) {
                case R.id.type_1:
                    feedback.setType(getString(R.string.feedback_type_1));
                    break;
                case R.id.type_2:
                    feedback.setType(getString(R.string.feedback_type_2));
                    break;
                case R.id.type_3:
                    feedback.setType(getString(R.string.feedback_type_3));
                    break;
                default:
                    feedback.setType(null);
            }
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (isChecked) {
            buttonView.setTextColor(ContextCompat.getColor(this, R.color.colorAccent));
        } else {
            buttonView.setTextColor(ContextCompat.getColor(this, android.R.color.black));
        }
        int problem;
        switch (buttonView.getId()) {
            case R.id.problem_1:
                problem = 0;
                break;
            case R.id.problem_2:
                problem = 1;
                break;
            case R.id.problem_3:
                problem = 2;
                break;
            case R.id.problem_4:
                problem = 3;
                break;
            case R.id.problem_5:
                problem = 4;
                break;
            case R.id.problem_6:
                problem = 5;
                break;
            case R.id.problem_7:
                problem = 6;
                break;
            default:
                problem = -1;
        }
        problems.put(problem, isChecked);
    }

    @Override
    public void beforeTextChanged(CharSequence s, int start, int count, int after) {
    }

    @Override
    public void onTextChanged(CharSequence s, int start, int before, int count) {
    }

    @Override
    public void afterTextChanged(Editable s) {
        feedback.setContent(s.toString());
        problemContentLimit.setText(String.format("%s / 200", s.length()));
    }
}
