package cn.minbb.evaluation.activities;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v4.widget.SwipeRefreshLayout;
import android.support.v7.app.AppCompatActivity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.adapter.PaperListAdapter;
import cn.minbb.evaluation.views.PullUpLoadMoreListView;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.dao.PaperDao;
import cn.minbb.evaluation.entity.Paper;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Response;

public class HistoryPaperActivity extends AppCompatActivity implements SwipeRefreshLayout.OnRefreshListener,
        PullUpLoadMoreListView.OnPullUpLoadMoreListener, AdapterView.OnItemClickListener {

    @BindView(R.id.refreshLayout)
    SwipeRefreshLayout refreshLayout;
    @BindView(R.id.papersView)
    PullUpLoadMoreListView papersView;

    private ProgressDialog dialog;

    private PaperDao paperDao = PaperDao.getPaperDao();
    private List<Paper> paperList = null;
    private PaperListAdapter paperListAdapter;
    private List<PaperListAdapter.ItemBean> itemBeanList = new ArrayList<>();

    int page;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_history_paper);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
        }
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, HistoryPaperActivity.class));
    }

    private void initView() {
        App.setupActionBar(this);

        refreshLayout.setColorSchemeResources(
                android.R.color.holo_blue_bright,
                android.R.color.holo_green_light,
                android.R.color.holo_orange_light,
                android.R.color.holo_red_light);
        showProgress(true);

        loadLocalData();
    }

    private void loadLocalData() {
        if (paperList == null) {
            paperList = new ArrayList<>();
            paperList = paperDao.findAll();
        }
        for (Paper paper : paperList) {
            itemBeanList.add(new PaperListAdapter.ItemBean(paper.getScore(), paper.getMark()));
        }
        paperListAdapter = new PaperListAdapter(this, itemBeanList);
        papersView.setAdapter(paperListAdapter);
    }

    private void setListener() {
        refreshLayout.setOnRefreshListener(this);
        papersView.setOnItemClickListener(this);
        papersView.setOnLoadMoreListener(this);
    }

    private void loadData() {
        if (!dialog.isShowing()) {
            refreshLayout.setRefreshing(true);
        }
        OkHttpClient client = new OkHttpClient();
        FormBody.Builder formBody = new FormBody.Builder();
        page = 0;
        formBody.add("page", page + "");
        formBody.add("size", "10");
        formBody.add("user", JSONObject.toJSONString(UserManager.getCurrentUser()));
        okhttp3.Request request = new okhttp3.Request.Builder()
                .url(Constant.SERVER_HOST + "papers")
                .post(formBody.build())
                .build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                HistoryPaperActivity.this.runOnUiThread(() -> {
                    if (refreshLayout.isRefreshing()) {
                        refreshLayout.setRefreshing(false);
                    }
                    showProgress(false);
                    Snackbar.make(getWindow().getDecorView(), "获取数据失败", Snackbar.LENGTH_SHORT).show();
                });
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                JSONObject jsonObject = JSONObject.parseObject(response.body().string());
                HistoryPaperActivity.this.runOnUiThread(() -> {
                    if (refreshLayout.isRefreshing()) {
                        refreshLayout.setRefreshing(false);
                    }
                    showProgress(false);

                    switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                        case Result.SUCCESS:
                            try {
                                List<Paper> paperListRecently = JSONArray.parseArray(jsonObject.getString(ResponseMap.DATA), Paper.class);
                                if (paperListRecently.size() == 0) {
                                    Snackbar.make(getWindow().getDecorView(), "没有记录", Snackbar.LENGTH_LONG).show();
                                } else {
                                    itemBeanList.clear();
                                    paperList.clear();
                                    paperList.addAll(paperListRecently);
                                    paperDao.deleteAll();
                                    paperDao.saveAll(paperListRecently);
                                    loadLocalData();
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                                Snackbar.make(getWindow().getDecorView(), "数据格式有误", Snackbar.LENGTH_LONG).show();
                            }
                            break;
                        case Result.FAILED:
                            Snackbar.make(getWindow().getDecorView(), "获取历史试卷失败！\n" + jsonObject.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                            break;
                        default:
                    }
                });
            }
        });
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setMessage("查询中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 0, 0, getString(R.string.action_info)).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        menu.findItem(0).setIcon(
                new IconDrawable(this, FontAwesomeIcons.fa_info_circle)
                        .colorRes(R.color.white)
                        .actionBarSize());
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                HistoryPaperActivity.this.finish();
                break;
            case 0:
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("提示");
                builder.setMessage("成绩和颜色按照百分比等级制评定显示，规则如下：");
                builder.setView(getLayoutInflater().inflate(R.layout.info_history_paper, null));
                builder.setPositiveButton(getString(R.string.action_ok), null);
                builder.create().show();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onRefresh() {
        loadData();
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        Paper paper = paperList.get(position);
        if (paper.getMark() != null) {
            MarkActivity.start(HistoryPaperActivity.this, JSONObject.toJSONString(paperList.get(position)));
        } else {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setTitle("提示")
                    .setMessage("点击“确定”继续进行之前未完成的评测内容")
                    .setPositiveButton(getString(R.string.operation_confirm), new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialog, int which) {
                        }
                    })
                    .setNegativeButton(getString(R.string.operation_cancel), null)
                    .create().show();
        }
    }

    @Override
    public void onLoadMore(PullUpLoadMoreListView pullUpLoadMoreListView) {
        OkHttpClient client = new OkHttpClient();
        FormBody.Builder formBody = new FormBody.Builder();
        formBody.add("page", ++page + "");
        formBody.add("size", "10");
        formBody.add("user", JSONObject.toJSONString(UserManager.getCurrentUser()));
        okhttp3.Request request = new okhttp3.Request.Builder()
                .url(Constant.SERVER_HOST + "papers")
                .post(formBody.build())
                .build();
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                HistoryPaperActivity.this.runOnUiThread(() -> {
                    papersView.loadComplete();
                    Snackbar.make(getWindow().getDecorView(), "获取数据失败", Snackbar.LENGTH_SHORT).show();
                });
            }

            @Override
            public void onResponse(Call call, Response response) throws IOException {
                JSONObject jsonObject = JSONObject.parseObject(response.body().string());
                HistoryPaperActivity.this.runOnUiThread(() -> {
                    papersView.loadComplete();
                    switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                        case Result.SUCCESS:
                            try {
                                List<Paper> paperListPart = JSONArray.parseArray(jsonObject.getString(ResponseMap.DATA), Paper.class);
                                if (paperListPart.size() == 0) {
                                    Snackbar.make(getWindow().getDecorView(), "没有更多记录", Snackbar.LENGTH_LONG).show();
                                } else {
                                    for (Paper paper : paperListPart) {
                                        itemBeanList.add(new PaperListAdapter.ItemBean(paper.getScore(), paper.getMark()));
                                    }
                                    Snackbar.make(getWindow().getDecorView(), String.format("获取了%d条数据", paperListPart.size()), Snackbar.LENGTH_LONG).show();
                                    paperListAdapter.notifyDataSetChanged();
                                    paperList.addAll(paperListPart);
                                    paperDao.saveAll(paperListPart);
                                }
                            } catch (Exception e) {
                                Snackbar.make(getWindow().getDecorView(), "数据格式有误", Snackbar.LENGTH_LONG).show();
                            }
                            break;
                        case Result.FAILED:
                            Snackbar.make(getWindow().getDecorView(), "获取历史试卷失败！\n" + jsonObject.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                            break;
                        default:
                    }
                });
            }
        });
    }
}
