package cn.minbb.evaluation.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.Snackbar;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.android.volley.Request;
import com.android.volley.toolbox.StringRequest;
import com.joanzapata.iconify.widget.IconTextView;

import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.evaluation.MainActivity;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.views.ClearTextInputEditText;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.User;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;

public class LoginActivity extends AppCompatActivity implements CompoundButton.OnCheckedChangeListener {

    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.help)
    FloatingActionButton help;
    @BindView(R.id.username)
    ClearTextInputEditText username;
    @BindView(R.id.password)
    ClearTextInputEditText password;
    @BindView(R.id.rememberPassword)
    CheckBox rememberPassword;
    @BindView(R.id.login)
    Button login;
    @BindView(R.id.registerAccount)
    TextView registerAccount;
    @BindView(R.id.losePassword)
    TextView losePassword;
    @BindView(R.id.serverStatus)
    TextView serverStatus;
    @BindView(R.id.serverStatusRefresh)
    IconTextView serverStatusRefresh;

    private ProgressDialog dialog;

    private boolean rememberPasswordIsChecked = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onRestart() {
        if (UserManager.getCurrentUser() != null) {
            LoginActivity.this.finish();
        }
        super.onRestart();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        App.getRequestQueue().cancelAll("login");
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
        }
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, LoginActivity.class));
    }

    private void initView() {
        toolbar.setNavigationIcon(R.drawable.ic_arrow_back_white_24dp);
        setSupportActionBar(toolbar);
    }

    private void setListener() {
        rememberPassword.setOnCheckedChangeListener(this);
    }

    private void loadData() {
        testServerStatus();
    }

    private void attemptLogin() {
        // 重置错误为空
        username.setError(null);
        password.setError(null);

        // 获取登录表单账号与密码
        String usernameText = username.getText().toString();
        String passwordText = password.getText().toString();

        boolean cancel = false;
        ClearTextInputEditText focusView = null;

        // 检查密码是否有效
        if (TextUtils.isEmpty(passwordText) || !isUserPasswordValid(passwordText)) {
            password.setError("密码至少六位");
            focusView = password;
            cancel = true;
        }

        // 检查用户名是否有效
        if (TextUtils.isEmpty(usernameText) || !isUsernameValid(usernameText)) {
            username.setError("用户名不能为空(不符合规则)");
            focusView = username;
            cancel = true;
        }

        if (cancel) {
            focusView.requestFocus();
            focusView.setShakeAnimation();
        } else {
            showProgress(true);
            toLogin(usernameText, passwordText);
        }
    }

    private boolean isUserPasswordValid(String passwordText) {
        return passwordText.length() >= 6;
    }

    private boolean isUsernameValid(String usernameText) {
        return usernameText.length() >= 1;
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("用户登录");
            dialog.setMessage("登录中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    private void toLogin(String usernameText, String passwordText) {
        StringRequest loginRequest = new StringRequest(Request.Method.POST,
                Constant.SERVER_HOST + "user/login",
                s -> {
                    showProgress(false);
                    try {
                        JSONObject jsonObject = JSONObject.parseObject(s);
                        switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                            case Result.SUCCESS:
                                UserManager.userLogin(com.alibaba.fastjson.JSONObject.parseObject(jsonObject.getString(ResponseMap.DATA), User.class), rememberPasswordIsChecked);
                                MainActivity.start(LoginActivity.this);
                                LoginActivity.this.finish();
                                break;
                            case Result.FAILED:
                                password.setText("");
                                Snackbar.make(toolbar.getRootView(), jsonObject.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                                break;
                            default:
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        Snackbar.make(toolbar.getRootView(), "获取到的数据有误！\n请检查互联网连接状态！", Snackbar.LENGTH_SHORT).show();
                    }
                },
                volleyError -> {
                    showProgress(false);
                    Snackbar.make(toolbar.getRootView(), "登录失败，请稍后重试！", Snackbar.LENGTH_SHORT).show();
                    Toast.makeText(LoginActivity.this, volleyError.getMessage(), Toast.LENGTH_SHORT).show();
                }) {
            @Override
            public Map<String, String> getHeaders() {
                Map<String, String> headers = new HashMap<>();
                headers.put("Content-Type", "application/x-www-form-urlencoded");
                return headers;
            }

            @Override
            protected Map<String, String> getParams() {
                Map<String, String> params = new HashMap<>();
                params.put("username", usernameText);
                params.put("password", passwordText);
                return params;
            }
        };
        loginRequest.setTag("login");
        App.getRequestQueue().add(loginRequest);
    }

    // 检查服务器状态
    public void testServerStatus() {
        serverStatusRefresh.setText("{fa-refresh spin}");
        StringRequest serverStatusRequest = new StringRequest(Request.Method.GET,
                Constant.SERVER_HOST,
                s -> {
                    serverStatusRefresh.setText("{fa-refresh}");
                    serverStatus.setText("在线");
                    serverStatus.setTextColor(ContextCompat.getColor(this, android.R.color.holo_green_light));
                },
                volleyError -> {
                    serverStatusRefresh.setText("{fa-refresh}");
                    serverStatus.setText("离线");
                    serverStatus.setTextColor(ContextCompat.getColor(this, android.R.color.holo_red_light));
                });
        App.getRequestQueue().add(serverStatusRequest);
    }

    private void exit() {
        AlertDialog.Builder builder = new AlertDialog.Builder(LoginActivity.this);
        builder.setTitle("取消登录？")
                .setMessage("取消登录将退出该系统。")
                .setPositiveButton(getString(R.string.operation_quit), (dialog, which) -> {
                    LoginActivity.this.finish();
                })
                .setNegativeButton(getString(R.string.operation_cancel), (dialog, which) -> {
                })
                .create().show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                exit();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @OnClick({R.id.help, R.id.login, R.id.registerAccount, R.id.losePassword, R.id.serverStatusRefresh})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.help:
                AlertDialog.Builder builder = new AlertDialog.Builder(LoginActivity.this);
                builder.setTitle("帮助")
                        .setMessage(getString(R.string.user_help))
                        .setPositiveButton("我知道啦", (dialog, which) -> {
                        })
                        .create().show();
                break;
            case R.id.login:
                attemptLogin();
                break;
            case R.id.registerAccount:
                RegisterActivity.start(this);
                break;
            case R.id.losePassword:
                Snackbar.make(toolbar.getRootView(), "密码丢失，请联系系统管理员", Snackbar.LENGTH_SHORT).show();
                break;
            case R.id.serverStatusRefresh:
                testServerStatus();
                break;
            default:
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        switch (buttonView.getId()) {
            case R.id.rememberPassword:
                rememberPasswordIsChecked = isChecked;
                break;
            default:
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            exit();
        }
        return super.onKeyDown(keyCode, event);
    }
}
