package cn.minbb.evaluation.activities;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.alibaba.fastjson.JSONObject;
import com.joanzapata.iconify.widget.IconTextView;

import org.reactivestreams.Subscriber;
import org.reactivestreams.Subscription;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.adapter.PublicChatAdapter;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.Message;
import cn.minbb.evaluation.entity.User;
import cn.minbb.evaluation.utils.Constant;
import io.reactivex.functions.Consumer;
import okhttp3.WebSocket;
import ua.naiksoftware.stomp.Stomp;
import ua.naiksoftware.stomp.client.StompClient;
import ua.naiksoftware.stomp.client.StompMessage;

public class PublicChatActivity extends AppCompatActivity {

    @BindView(R.id.publicChatMessageView)
    RecyclerView publicChatMessageView;
    @BindView(R.id.newMessageTip)
    TextView newMessageTip;
    @BindView(R.id.more)
    IconTextView more;
    @BindView(R.id.message)
    EditText message;
    @BindView(R.id.send)
    IconTextView send;

    private StompClient stompClient;
    private Timer reconnectStomp;
    private User user;
    private SimpleDateFormat simpleDateFormat = (SimpleDateFormat) SimpleDateFormat.getTimeInstance();

    private PublicChatAdapter publicChatAdapter;
    private List<PublicChatAdapter.MessageItemBean> messageItemBeanList = new ArrayList<>();
    private int newMessageCount = 0, hideMessageCount;

    static {
        AppCompatDelegate.setCompatVectorFromResourcesEnabled(true);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_public_chat);
        ButterKnife.bind(this);
        user = UserManager.getCurrentUser();

        initView();
        loadData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        // 取消自动重连并断开 WebSocket 长连接
        if (reconnectStomp != null) {
            reconnectStomp.cancel();
        }
        if (stompClient != null && stompClient.isConnected()) {
            stompClient.disconnect();
        }
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, PublicChatActivity.class);
        context.startActivity(starter);
    }

    private void initView() {
        if (user == null) {
            Toast.makeText(this, "抱歉，用户身份识别失败，请重新登录后重试。", Toast.LENGTH_SHORT).show();
            PublicChatActivity.this.finish();
        }
        App.setupActionBar(this);

        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测simpleDateFormat.format(App.getCurrentDateTime())simpleDateFormat.format(App.getCurrentDateTime())试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(0, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "------ 系统消息 ------", "ok"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测simpleDateFormat.format(App.getCurrentDateTime())simpleDateFormat.format(App.getCurrentDateTime())simpleDateFormat.format(App.getCurrentDateTime())试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(2, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测simpleDateFormat.format(App.getCurrentDateTime())simpleDateFormat.format(App.getCurrentDateTime())试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(2, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(1, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(2, "", simpleDateFormat.format(App.getCurrentDateTime()), "Name", "测试", "{ion-ios-checkmark-outline}"));
        publicChatAdapter = new PublicChatAdapter(this, messageItemBeanList);
        publicChatAdapter.setOnItemLongClickListener(this::onItemLongClick);
        publicChatAdapter.setOnItemDoubleClickListener(this::onItemDoubleClick);
        publicChatMessageView.setAdapter(publicChatAdapter);
        publicChatMessageView.setLayoutManager(new LinearLayoutManager(this));
        publicChatMessageView.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrollStateChanged(RecyclerView recyclerView, int newState) {
                super.onScrollStateChanged(recyclerView, newState);
            }

            @Override
            public void onScrolled(RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                // 手动滑动到底部
                if (recyclerView.computeVerticalScrollExtent() + recyclerView.computeVerticalScrollOffset() >= recyclerView.computeVerticalScrollRange()) {
                    newMessageCount = 0;
                    newMessageTip.setVisibility(View.INVISIBLE);
                } else {
                    // 向下滑动
                    if (dy > 0) {
                        // 可见 Item 位于所有项中的位置
                        int itemPosition = recyclerView.getChildAdapterPosition(recyclerView.findChildViewUnder(dx, dy));
                        if (itemPosition > 2) {
                            hideMessageCount = messageItemBeanList.size() - itemPosition;
                            if (newMessageCount > hideMessageCount) {
                                newMessageCount = hideMessageCount;
                                newMessageTip.setText(String.format("%s", newMessageCount));
                            }
                        }
                    }
                }
            }
        });
    }

    private void loadData() {
        initSocket();
    }

    @SuppressLint("CheckResult")
    private void initSocket() {
        if (stompClient == null) {
            stompClient = Stomp.over(WebSocket.class, Constant.SERVER_WS_HOST);
            stompClient.connect();
            stompClient.lifecycle().subscribe(lifecycleEvent -> {
                switch (lifecycleEvent.getType()) {
                    case OPENED:
                        System.out.println("连接已开启");
                        if (reconnectStomp != null) {
                            reconnectStomp.cancel();
                        }
                        break;
                    case ERROR:
                        System.out.println("连接出错 = " + lifecycleEvent.getException());
                        reconnectSocket();
                        break;
                    case CLOSED:
                        System.out.println("连接已关闭");
                        reconnectSocket();
                        break;
                    default:
                }
            });
            // 订阅公共聊天室消息
            stompClient.topic("/topic/chat/all").subscribe(new Consumer<StompMessage>() {
                @Override
                public void accept(StompMessage stompMessage) {
                    runOnUiThread(() -> {
                        try {
                            Message message = JSONObject.parseObject(stompMessage.getPayload(), Message.class);
                            if (message != null) {
                                User userFrom = message.getUserFrom();
                                PublicChatAdapter.MessageItemBean messageItemBean;
                                if (userFrom != null) {
                                    messageItemBean = new PublicChatAdapter.MessageItemBean(
                                            PublicChatAdapter.RECEIVE,
                                            userFrom.getAvatar(),
                                            simpleDateFormat.format(message.getCreatedAt()),
                                            userFrom.getName(),
                                            message.getContent(),
                                            "OK");
                                } else {
                                    messageItemBean = new PublicChatAdapter.MessageItemBean(
                                            PublicChatAdapter.RECEIVE,
                                            null,
                                            simpleDateFormat.format(message.getCreatedAt()),
                                            null,
                                            message.getContent(),
                                            "OK");
                                }
                                messageItemBeanList.add(messageItemBean);
                                publicChatAdapter.notifyDataSetChanged();
                                // RecyclerView.canScrollVertically(1)的值表示是否能向上滚动，false表示已经滚动到底部
                                // RecyclerView.canScrollVertically(-1)的值表示是否能向下滚动，false表示已经滚动到顶部
                                if (!publicChatMessageView.canScrollVertically(1)) {
                                    publicChatMessageView.smoothScrollToPosition(publicChatAdapter.getItemCount() - 1);
                                } else {
                                    // 提示收到新消息
                                    newMessageCount++;
                                    newMessageTip.setVisibility(View.VISIBLE);
                                    newMessageTip.setText(String.format("%s", newMessageCount));
                                }
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    });
                }
            });
        }
    }

    /**
     * 重新连接 WebSocket
     */
    private void reconnectSocket() {
        if (stompClient != null) {
            stompClient.disconnect();
            stompClient = null;
        }
        reconnectStomp = new Timer();
        reconnectStomp.schedule(new TimerTask() {
            @Override
            public void run() {
                initSocket();
                reconnectStomp.cancel();
            }
        }, 5 * 1000);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                PublicChatActivity.this.finish();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @OnClick({R.id.newMessageTip, R.id.more, R.id.send})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.newMessageTip:
                newMessageTip.setVisibility(View.VISIBLE);
                publicChatMessageView.smoothScrollToPosition(publicChatAdapter.getItemCount() - newMessageCount);
                newMessageCount -= 5;
                newMessageTip.setText(String.format("%s", newMessageCount));
                break;
            case R.id.more:
                break;
            case R.id.send:
                if (stompClient != null && stompClient.isConnected()) {
                    Message messageSend = new Message();
                    messageSend.setUserFrom(user);
                    messageSend.setContent(message.getText().toString());

                    stompClient.send("/app/chat/all" + 1, JSONObject.toJSONString(messageSend)).subscribe(new Subscriber<Void>() {
                        @Override
                        public void onSubscribe(Subscription s) {
                            System.out.println("订阅 = " + s.toString());
                        }

                        @Override
                        public void onNext(Void aVoid) {
                            System.out.println("下一步");
                        }

                        @Override
                        public void onError(Throwable t) {
                            System.out.println("出错 = " + t.getStackTrace());
                        }

                        @Override
                        public void onComplete() {
                            System.out.println("发送完成");
                        }
                    });
                    message.setText("");
                    messageItemBeanList.add(new PublicChatAdapter.MessageItemBean(PublicChatAdapter.SEND, "", App.getCurrentStringTime(), user.getName(), messageSend.getContent(), "{ion-ios-refresh-outline spin}"));
                    publicChatAdapter.notifyDataSetChanged();
                    publicChatMessageView.smoothScrollToPosition(publicChatAdapter.getItemCount() - 1);
                }
                break;
        }
    }

    public void onItemLongClick(View view, int position) {
        Toast.makeText(this, "" + position, Toast.LENGTH_SHORT).show();
    }

    public void onItemDoubleClick(View view, int position) {
        Toast.makeText(this, ((TextView) (view.findViewById(R.id.messageText))).getText().toString(), Toast.LENGTH_SHORT).show();
    }
}
