package cn.minbb.evaluation.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.CompoundButton;
import android.widget.SeekBar;

import com.alibaba.fastjson.JSONObject;
import com.joanzapata.iconify.IconDrawable;
import com.joanzapata.iconify.fonts.FontAwesomeIcons;
import com.joanzapata.iconify.widget.IconTextView;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.adapter.CardItem;
import cn.minbb.evaluation.adapter.CardPagerAdapter;
import cn.minbb.evaluation.adapter.ShadowTransformer;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.Answer;
import cn.minbb.evaluation.entity.Paper;
import cn.minbb.evaluation.entity.Question;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Response;

public class QuestionActivity extends AppCompatActivity implements SeekBar.OnSeekBarChangeListener,
        ViewPager.OnPageChangeListener, CardPagerAdapter.OnAnswerChangedListener {

    @BindView(R.id.progress)
    SeekBar progress;
    @BindView(R.id.progressText)
    IconTextView progressText;
    @BindView(R.id.viewPager)
    ViewPager viewPager;

    private ProgressDialog dialog;

    private Paper paperSubmit;
    private List<Question> questionList, questionListSubmit;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_question);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    public static void start(Context context, String paperJSON) {
        Intent intent = new Intent(context, QuestionActivity.class);
        intent.putExtra("paperJSON", paperJSON);
        context.startActivity(intent);
    }

    private void initView() {
        App.setupActionBar(this);
    }

    private void setListener() {
        progress.setOnSeekBarChangeListener(this);
        viewPager.addOnPageChangeListener(this);
    }

    private void loadData() {
        String paperJSON = getIntent().getStringExtra("paperJSON");
        Paper paper = JSONObject.parseObject(paperJSON, Paper.class);
        questionList = paper.getQuestionList();
        // 给要提交的试卷写入 ID，初始化试卷问题列表
        paperSubmit = new Paper(paper.getE_id());
        questionListSubmit = new ArrayList<>(questionList.size());
        for (Question question : questionList) {
            questionListSubmit.add(new Question(question.getE_id()));
        }
        paperSubmit.setQuestionList(questionListSubmit);

        progress.setMax(questionList.size());
        progress.setProgress(1);

        CardPagerAdapter cardPagerAdapter = new CardPagerAdapter(QuestionActivity.this);
        for (int i = 0; i < questionList.size(); i++) {
            cardPagerAdapter.addCardItem(new CardItem(i, questionList.get(i)));
        }

        ShadowTransformer shadowTransformer = new ShadowTransformer(viewPager, cardPagerAdapter);
        shadowTransformer.enableScaling(true);
        cardPagerAdapter.setOnAnswerChangedListener(this);
        viewPager.setAdapter(cardPagerAdapter);
        viewPager.setPageTransformer(false, shadowTransformer);
        viewPager.setOffscreenPageLimit(3);
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("提交");
            dialog.setMessage("正在提交试卷中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    private void backToHome() {
        AlertDialog.Builder home = new AlertDialog.Builder(this);
        home.setTitle("退出");
        home.setMessage("确定退出答题吗？\n系统不进行评测成绩，将在下次完成该评测内容时评测成绩。");
        home.setNegativeButton(getString(R.string.operation_cancel), null);
        home.setPositiveButton(getString(R.string.operation_quit), (dialog, which) -> QuestionActivity.this.finish());
        home.create().show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        menu.add(0, 0, 0, getString(R.string.operation_submit)).setShowAsAction(MenuItem.SHOW_AS_ACTION_ALWAYS);
        // Set an icon in the ActionBar
        menu.findItem(0).setIcon(
                new IconDrawable(this, FontAwesomeIcons.fa_cloud_upload)
                        .colorRes(R.color.white)
                        .actionBarSize());
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                backToHome();
                break;
            case 0:
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("提交？")
                        .setMessage("交卷后将无法法修改，确定要提交试卷吗？（系统将在线评定成绩并第一时间将结果反馈给您）")
                        .setPositiveButton(getString(R.string.operation_submit), (dialog, which) -> {
                            showProgress(true);
                            OkHttpClient client = new OkHttpClient();
                            FormBody.Builder formBody = new FormBody.Builder();
                            formBody.add("paper", JSONObject.toJSONString(paperSubmit));
                            formBody.add("user", JSONObject.toJSONString(UserManager.getCurrentUser()));
                            okhttp3.Request request = new okhttp3.Request.Builder()
                                    .url(Constant.SERVER_HOST + "question/evaluate")
                                    .post(formBody.build())
                                    .build();
                            client.newCall(request).enqueue(new Callback() {
                                @Override
                                public void onFailure(Call call, IOException e) {
                                    QuestionActivity.this.runOnUiThread(() -> {
                                        showProgress(false);
                                        Snackbar.make(getWindow().getDecorView(), "获取数据失败", Snackbar.LENGTH_SHORT).show();
                                    });
                                }

                                @Override
                                public void onResponse(Call call, Response response) throws IOException {
                                    JSONObject jsonObject = JSONObject.parseObject(response.body().string());
                                    QuestionActivity.this.runOnUiThread(() -> {
                                        showProgress(false);
                                        switch (jsonObject.getInteger(ResponseMap.RESULT)) {
                                            case Result.SUCCESS:
                                                MarkActivity.start(QuestionActivity.this, jsonObject.getString(ResponseMap.DATA));
                                                QuestionActivity.this.finish();
                                                break;
                                            case Result.FAILED:
                                                Snackbar.make(getWindow().getDecorView(), "在线打分失败\n" + jsonObject.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                                                break;
                                            default:
                                        }
                                    });
                                }
                            });
                        }).setNegativeButton(getString(R.string.operation_cancel), null).create().show();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
        progressText.setText(progress + " / " + questionList.size());
        viewPager.setCurrentItem(progress - 1);
    }

    @Override
    public void onStartTrackingTouch(SeekBar seekBar) {
    }

    @Override
    public void onStopTrackingTouch(SeekBar seekBar) {
    }

    @Override
    public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
    }

    @Override
    public void onPageSelected(int position) {
        progress.setProgress(position + 1);
    }

    @Override
    public void onPageScrollStateChanged(int state) {
    }

    @Override
    public void onAnswerChanged(CompoundButton buttonView, boolean isChecked, CardItem item) {
        if (isChecked) {
            Set<Answer> answerSet = new HashSet<>(1);
            answerSet.add(new Answer((Long) buttonView.getTag()));
            questionListSubmit.get(item.getPosition()).setAnswerSet(answerSet);
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            backToHome();
        }
        return super.onKeyDown(keyCode, event);
    }
}
