package cn.minbb.evaluation.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.CycleInterpolator;
import android.view.animation.TranslateAnimation;
import android.widget.CheckBox;
import android.widget.CompoundButton;

import com.android.volley.Request;
import com.android.volley.toolbox.JsonObjectRequest;

import org.json.JSONObject;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.evaluation.MainActivity;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.views.ClearTextInputEditText;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.User;
import cn.minbb.evaluation.utils.Constant;
import cn.minbb.evaluation.utils.ResponseMap;
import cn.minbb.evaluation.utils.Result;
import cn.minbb.evaluation.utils.StringMap;

public class RegisterActivity extends AppCompatActivity implements CompoundButton.OnCheckedChangeListener {

    @BindView(R.id.toolbar)
    Toolbar toolbar;
    @BindView(R.id.username)
    ClearTextInputEditText username;
    @BindView(R.id.password)
    ClearTextInputEditText password;
    @BindView(R.id.repeatPassword)
    ClearTextInputEditText repeatPassword;
    @BindView(R.id.alreadyRead)
    CheckBox alreadyRead;

    private boolean alreadyReadIsChecked = false;
    private ProgressDialog dialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_register);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        App.getRequestQueue().cancelAll("register");
        if (dialog != null && dialog.isShowing()) {
            dialog.dismiss();
        }
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, RegisterActivity.class));
    }

    private void initView() {
        toolbar.setNavigationIcon(R.drawable.ic_arrow_back_white_24dp);
        setSupportActionBar(toolbar);
    }

    private void setListener() {
        alreadyRead.setOnCheckedChangeListener(this);
    }

    private void loadData() {
    }

    private void attemptRegister() {
        // 重置错误为空
        username.setError(null);
        password.setError(null);
        repeatPassword.setError(null);

        // 获取登录表单账号与密码
        String usernameText = username.getText().toString();
        String passwordText = password.getText().toString();
        String repeatPasswordText = repeatPassword.getText().toString();

        boolean cancel = false;
        View focusView = null;

        // 已阅用户须知
        if (!alreadyReadIsChecked) {
            focusView = alreadyRead;
            cancel = true;
            Snackbar.make(toolbar.getRootView(), "请先阅读用户须知", Snackbar.LENGTH_SHORT).show();
        }

        // 检查密码是否有效
        if (TextUtils.isEmpty(passwordText) || !isUserPasswordValid(passwordText)) {
            password.setError("密码位数在 6 ~ 32 之间");
            focusView = password;
            cancel = true;
        } else if (!passwordText.equals(repeatPasswordText)) {
            repeatPassword.setError("两次输入的密码不一致");
            focusView = repeatPassword;
            cancel = true;
        }

        // 检查用户名是否有效
        if (TextUtils.isEmpty(usernameText) || !isUsernameValid(usernameText)) {
            username.setError("用户名不能为空(不符合规则)");
            focusView = username;
            cancel = true;
        }

        if (cancel) {
            assert focusView != null;
            focusView.requestFocus();
            Animation translateAnimation = new TranslateAnimation(0, 10, 0, 0);
            translateAnimation.setInterpolator(new CycleInterpolator(5));
            translateAnimation.setDuration(1000);
            focusView.setAnimation(translateAnimation);
        } else {
            showProgress(true);
            toRegister(usernameText, passwordText);
        }
    }

    private boolean isUserPasswordValid(String passwordText) {
        int length = passwordText.length();
        return length >= 6 && length <= 32;
    }

    private boolean isUsernameValid(String usernameText) {
        return usernameText.length() >= 1;
    }

    private void showProgress(boolean show) {
        if (dialog == null) {
            dialog = new ProgressDialog(this);
            dialog.setTitle("用户注册");
            dialog.setMessage("注册中······");
            dialog.setCancelable(false);
        }
        if (show) {
            dialog.show();
        } else {
            dialog.dismiss();
        }
    }

    private void toRegister(String usernameText, String passwordText) {
        JsonObjectRequest registerRequest = new JsonObjectRequest(
                Request.Method.POST,
                Constant.SERVER_HOST + "user/register",
                new JSONObject(new StringMap(2).put("username", usernameText).put("password", passwordText).getMap()),
                jsonObject -> {
                    showProgress(false);
                    com.alibaba.fastjson.JSONObject json = com.alibaba.fastjson.JSONObject.parseObject(jsonObject.toString());
                    switch (json.getInteger(ResponseMap.RESULT)) {
                        case Result.SUCCESS:
                            try {
                                Snackbar.make(toolbar.getRootView(), "注册成功\n3s 后将自动登录", Snackbar.LENGTH_LONG).show();
                                UserManager.userLogin(com.alibaba.fastjson.JSONObject.parseObject(json.getString(ResponseMap.DATA), User.class), true);
                                new Handler().postDelayed(() -> {
                                    MainActivity.start(RegisterActivity.this);
                                    RegisterActivity.this.finish();
                                }, 3000);
                            } catch (Exception e) {
                                Snackbar.make(toolbar.getRootView(), "注册成功。\n但是由于用户数据格式错误导致自动登录失败！", Snackbar.LENGTH_SHORT).show();
                                new Handler().postDelayed(() -> {
                                    LoginActivity.start(RegisterActivity.this);
                                    RegisterActivity.this.finish();
                                }, 3000);
                            }
                            break;
                        case Result.FAILED:
                            Snackbar.make(toolbar.getRootView(), json.getString(ResponseMap.SUMMARY), Snackbar.LENGTH_LONG).show();
                            break;
                        default:
                    }
                },
                volleyError -> {
                    showProgress(false);
                    Snackbar.make(toolbar.getRootView(), "注册失败，请稍后重试！", Snackbar.LENGTH_SHORT).show();
                });
        registerRequest.setTag("register");
        App.getRequestQueue().add(registerRequest);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @OnClick({R.id.userInstructions, R.id.register})
    public void onViewClicked(View view) {
        switch (view.getId()) {
            case R.id.userInstructions:
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("用户须知")
                        .setMessage(getString(R.string.user_instructions))
                        .setPositiveButton("同意以上条款", (dialog, which) -> {
                            alreadyRead.setChecked(true);
                        })
                        .setNegativeButton("不同意以上条款", (dialog, which) -> {
                            alreadyRead.setChecked(false);
                        })
                        .setCancelable(false).create().show();
                break;
            case R.id.register:
                attemptRegister();
                break;
        }
    }

    @Override
    public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
        if (buttonView.getId() == R.id.alreadyRead) {
            alreadyReadIsChecked = isChecked;
        }
    }
}
