package cn.minbb.evaluation.activities;

import android.annotation.TargetApi;
import android.content.Context;
import android.content.Intent;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceFragment;
import android.preference.PreferenceManager;
import android.preference.RingtonePreference;
import android.support.v7.app.ActionBar;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import java.util.List;

import cn.minbb.evaluation.R;

/**
 * 提供了一组应用程序设置。在手机设备上，设置将显示为单个列表。在平板电脑上，设置按类别拆分，类别标题显示在设置列表的左侧。
 */
public class SettingsActivity extends AppCompatPreferenceActivity implements Preference.OnPreferenceClickListener {

    // 更改首选项摘要以反映其新值的首选项值更改侦听器。
    private static Preference.OnPreferenceChangeListener bindPreferenceSummaryToValueListener = new Preference.OnPreferenceChangeListener() {
        @Override
        public boolean onPreferenceChange(Preference preference, Object newValue) {
            String stringValue = newValue.toString();
            if (preference instanceof ListPreference) {
                // 对于列表首选项，请在首选项的“条目”列表中查找正确的显示值。
                ListPreference listPreference = (ListPreference) preference;
                int index = listPreference.findIndexOfValue(stringValue);
            } else if (preference instanceof RingtonePreference) {
                // 对于铃声首选项，使用铃声管理器查找正确的显示值。
                if (TextUtils.isEmpty(stringValue)) {
                    // 空值对应于'无声'（无铃声）。
                    preference.setSummary(R.string.pref_ringtone_silent);
                } else {
                    Ringtone ringtone = RingtoneManager.getRingtone(preference.getContext(), Uri.parse(stringValue));
                    if (ringtone == null) {
                        // 如果发现查找错误，请清除摘要。
                        preference.setSummary(null);
                    } else {
                        // 设置摘要以反映新的铃声显示名称。
                        String name = ringtone.getTitle(preference.getContext());
                        preference.setSummary(name);
                    }
                }
            } else {
                // 对于所有其他首选项，请将摘要设置为值的简单字符串表示形式。
                preference.setSummary(stringValue);
            }
            return true;
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setupActionBar();

        addPreferencesFromResource(R.xml.pref_settings);

        // bindPreferenceSummaryToValue(findPreference("check_update_auto"));
        findPreference("check_update_now").setOnPreferenceClickListener(this);
    }

    private void setupActionBar() {
        ViewGroup rootView = findViewById(R.id.action_bar_root);
        if (rootView != null) {
            View view = getLayoutInflater().inflate(R.layout.app_bar_layout, rootView, false);
            rootView.addView(view, 0);

            Toolbar toolbar = findViewById(R.id.toolbar);
            setSupportActionBar(toolbar);
        }

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            // 在操作栏中显示向上按钮。
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, SettingsActivity.class);
        context.startActivity(starter);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
            SettingsActivity.this.finish();
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onPreferenceClick(Preference preference) {
        switch (preference.getKey()) {
            case "check_update_now":
                Toast.makeText(this, "立即检查更新", Toast.LENGTH_SHORT).show();
                break;
            default:
        }
        return false;
    }

    /**
     * 将首选项摘要绑定到其值。更具体地说，当首选项的值发生变化时，其摘要（首选项标题下面的文本行）将更新以反映该值。摘要也会在调用此方法后立即更新。确切的显示格式取决于首选项的类型。
     */
    private static void bindPreferenceSummaryToValue(Preference preference) {
        // 设置监听器以监视值更改。
        preference.setOnPreferenceChangeListener(bindPreferenceSummaryToValueListener);
        // 立即使用首选项的当前值触发侦听器。
        bindPreferenceSummaryToValueListener.onPreferenceChange(preference, PreferenceManager.getDefaultSharedPreferences(preference.getContext()).getString(preference.getKey(), ""));
    }

    @Override
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public void onBuildHeaders(List<Header> target) {
        // loadHeadersFromResource(R.xml.pref_settings, target);
    }

    /**
     * 此方法停止恶意应用程序中的片段注入。确保在这里否认任何未知的片段。
     */
    protected boolean isValidFragment(String fragmentName) {
        return PreferenceFragment.class.getName().equals(fragmentName)
                || GeneralPreferenceFragment.class.getName().equals(fragmentName)
                || DataSyncPreferenceFragment.class.getName().equals(fragmentName)
                || NotificationPreferenceFragment.class.getName().equals(fragmentName);
    }

    public boolean onPreferenceChange(Preference preference, Object newValue) {
        String stringValue = newValue.toString();
        if (preference instanceof ListPreference) {
            // 对于列表首选项，请在首选项的“条目”列表中查找正确的显示值。
            ListPreference listPreference = (ListPreference) preference;
            int index = listPreference.findIndexOfValue(stringValue);
            // 设置摘要以反映新值。
            preference.setSummary(index >= 0 ? listPreference.getEntries()[index] : null);
        } else if (preference instanceof RingtonePreference) {
            // 对于铃声首选项，使用铃声管理器查找正确的显示值。
            if (TextUtils.isEmpty(stringValue)) {
                // 空值对应于'无声'（无铃声）。
                preference.setSummary(R.string.pref_ringtone_silent);
            } else {
                Ringtone ringtone = RingtoneManager.getRingtone(preference.getContext(), Uri.parse(stringValue));
                if (ringtone == null) {
                    // 如果发现查找错误，请清除摘要。
                    preference.setSummary(null);
                } else {
                    // 设置摘要以反映新的铃声显示名称。
                    String name = ringtone.getTitle(preference.getContext());
                    preference.setSummary(name);
                }
            }
        } else {
            // 对于所有其他首选项，请将摘要设置为值的简单字符串表示形式。
            preference.setSummary(stringValue);
        }
        return true;
    }

    /**
     * 该片段仅显示一般偏好。它在活动显示双窗格设置UI时使用。
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class GeneralPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.pref_general);
            setHasOptionsMenu(true);

            // 将 EditText / List / Dialog / Ringtone 首选项的摘要绑定到它们的值。
            // 当它们的值发生变化时，根据 Android 设计指南，它们的摘要会更新以反映新值。
            bindPreferenceSummaryToValue(findPreference("example_text"));
            bindPreferenceSummaryToValue(findPreference("example_list"));
        }

        @Override
        public boolean onOptionsItemSelected(MenuItem item) {
            int id = item.getItemId();
            if (id == android.R.id.home) {
                startActivity(new Intent(getActivity(), SettingsActivity.class));
                getFragmentManager().popBackStack();
                return true;
            }
            return super.onOptionsItemSelected(item);
        }
    }

    /**
     * 此片段仅显示通知首选项。它在活动显示双窗格设置UI时使用。
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class NotificationPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.pref_notification);
            setHasOptionsMenu(true);

            bindPreferenceSummaryToValue(findPreference("notifications_new_message_ringtone"));
        }

        @Override
        public boolean onOptionsItemSelected(MenuItem item) {
            int id = item.getItemId();
            if (id == android.R.id.home) {
                startActivity(new Intent(getActivity(), SettingsActivity.class));
                return true;
            }
            return super.onOptionsItemSelected(item);
        }
    }

    /**
     * 此片段仅显示数据和同步首选项。它在活动显示双窗格设置UI时使用。
     */
    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
    public static class DataSyncPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            addPreferencesFromResource(R.xml.pref_data_sync);
            setHasOptionsMenu(true);

            bindPreferenceSummaryToValue(findPreference("sync_frequency"));
        }

        @Override
        public boolean onOptionsItemSelected(MenuItem item) {
            int id = item.getItemId();
            if (id == android.R.id.home) {
                startActivity(new Intent(getActivity(), SettingsActivity.class));
                return true;
            }
            return super.onOptionsItemSelected(item);
        }
    }
}
