package cn.minbb.evaluation.activities;

import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.view.KeyEvent;
import android.view.View;

import cn.minbb.evaluation.MainActivity;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.fragment.GuideFragment;

public class StartActivity extends AppCompatActivity {

    private View view;
    private Handler handler;
    private Runnable runnable;

    private int startCode;
    boolean fragmentOpen = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_start);

        SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);
        startCode = sharedPreferences.getInt("start_code", 0);

        handler = new Handler();
        runnable = () -> runOnUiThread(() -> {
            App.user = UserManager.getCurrentUser();
            if (App.user != null) {
                MainActivity.start(StartActivity.this);
            } else {
                LoginActivity.start(StartActivity.this);
            }
            StartActivity.this.finish();
            overridePendingTransition(R.anim.main_activity_in, 0);
        });

        initView();
        setListener();
        loadData();
    }

    private void initView() {
        view = findViewById(R.id.fullscreen_content);
    }

    private void setListener() {
    }

    private void loadData() {
        int versionCode = 0;
        try {
            versionCode = getPackageManager().getPackageInfo(getPackageName(), 0).versionCode;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        if (startCode != versionCode) {
            GuideFragment guideFragment = new GuideFragment();
            // 传参
            Bundle bundle = new Bundle();
            bundle.putInt("versionCode", versionCode);
            guideFragment.setArguments(bundle);
            FragmentManager fragmentManager = getSupportFragmentManager();
            fragmentManager.addOnBackStackChangedListener(() -> {
                if (fragmentOpen) {
                    fragmentOpen = false;
                    handler.postDelayed(runnable, 500);
                } else {
                    fragmentOpen = true;
                }
            });
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();
            // 设置 Fragment 动画
            fragmentTransaction
                    .setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out)
                    .addToBackStack(null)
                    .add(R.id.container, guideFragment, "")
                    .commit();
        } else {
            handler.postDelayed(runnable, 500);
        }
    }

    @Override
    protected void onPostCreate(Bundle savedInstanceState) {
        super.onPostCreate(savedInstanceState);

        ActionBar actionBar = getSupportActionBar();
        if (actionBar != null) {
            actionBar.hide();
        }
        view.setSystemUiVisibility(View.SYSTEM_UI_FLAG_LOW_PROFILE
                | View.SYSTEM_UI_FLAG_FULLSCREEN
                | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION);
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        handler.removeCallbacks(runnable);
    }

    // 屏蔽返回键
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return keyCode == KeyEvent.KEYCODE_BACK || super.onKeyDown(keyCode, event);
    }
}
