package cn.minbb.evaluation.activities;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.support.design.widget.Snackbar;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.DefaultRetryPolicy;
import com.android.volley.Request;
import com.android.volley.toolbox.StringRequest;
import com.bumptech.glide.Glide;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import cn.minbb.androidimagepicker.AndroidImagePicker;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.ctrl.UserManager;
import cn.minbb.evaluation.entity.User;
import cn.minbb.evaluation.utils.BitmapUtil;
import cn.minbb.evaluation.utils.Constant;

public class UserInfoActivity extends AppCompatActivity {

    @BindView(R.id.usernameText)
    TextView usernameText;
    @BindView(R.id.avatarImage)
    ImageView avatarImage;
    @BindView(R.id.nameText)
    TextView nameText;
    @BindView(R.id.mailText)
    TextView mailText;
    @BindView(R.id.phoneText)
    TextView phoneText;
    @BindView(R.id.qqText)
    TextView qqText;

    private User user;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_info);
        ButterKnife.bind(this);

        initView();
        setListener();
        loadData();
    }

    @Override
    protected void onDestroy() {
        // TODO: 2018-03-16 图片拾取器模块
        // AndroidImagePicker.getInstance().onDestroy();
        super.onDestroy();
    }

    public static void start(Context context) {
        context.startActivity(new Intent(context, UserInfoActivity.class));
    }

    private void initView() {
        App.setupActionBar(this);
    }

    private void setListener() {
    }

    private void loadData() {
        user = UserManager.getCurrentUser();
        if (user == null) {
            usernameText.setText("");
            avatarImage.setImageDrawable(ContextCompat.getDrawable(this, R.mipmap.ic_launcher));
            nameText.setText("");
            mailText.setText("");
            phoneText.setText("");
            qqText.setText("");
        } else {
            usernameText.setText(user.getUsername());
            Glide.with(this)
                    .load(user.getAvatar())
                    .placeholder(R.mipmap.ic_launcher)
                    .error(R.mipmap.ic_launcher)
                    .into(avatarImage);
            nameText.setText(user.getName());
            mailText.setText(user.getEmail());
            phoneText.setText(user.getPhone());
            qqText.setText(user.getQq());
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                UserInfoActivity.this.finish();
                break;
            default:
        }
        return super.onOptionsItemSelected(item);
    }

    @OnClick({R.id.avatar, R.id.name, R.id.mail, R.id.phone, R.id.qq, R.id.alertPassword, R.id.logout})
    public void onViewClicked(View view) {
        if (user != null) {
            switch (view.getId()) {
                case R.id.avatar:
                    Toast.makeText(this, "Avatar", Toast.LENGTH_SHORT).show();
                    AndroidImagePicker.getInstance().pickAndCrop(UserInfoActivity.this, true, 640, (bmp, ratio) -> {
                        ImageView avatar = new ImageView(this);
                        avatar.setImageBitmap(bmp);
                        avatar.setPadding(48, 24, 48, 0);
                        new AlertDialog.Builder(this)
                                .setTitle("头像预览")
                                .setView(avatar)
                                .setPositiveButton(getString(R.string.operation_confirm), new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        ProgressDialog progressDialog = ProgressDialog.show(
                                                UserInfoActivity.this,
                                                "请稍后",
                                                "上传头像中······",
                                                false,
                                                false);
                                        // 表单形式上传头像
                                        StringRequest avatarRequest = new StringRequest(Request.Method.POST,
                                                Constant.SERVER_HOST + "user/alter/avatar",
                                                s -> {
                                                    progressDialog.cancel();
                                                    System.out.println("response = " + s);
                                                    Toast.makeText(UserInfoActivity.this, "response = " + s, Toast.LENGTH_SHORT).show();
                                                },
                                                volleyError -> {
                                                    progressDialog.cancel();
                                                    System.out.println("error = " + volleyError.getMessage());
                                                    Toast.makeText(UserInfoActivity.this, "error = " + volleyError.getMessage(), Toast.LENGTH_SHORT).show();
                                                }) {
                                            @Override
                                            public Map<String, String> getHeaders() {
                                                Map<String, String> headers = new HashMap<>();
                                                headers.put("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8");
                                                headers.put("Accept-Encoding", "gzip, deflate, br");
                                                return headers;
                                            }

                                            @Override
                                            protected Map<String, String> getParams() throws AuthFailureError {
                                                return super.getParams();
                                            }

                                            @Override
                                            public byte[] getBody() {
                                                String splitLine = "------WebKitFormBoundaryYumin";
                                                StringBuilder sb = new StringBuilder();
                                                sb.append(splitLine).append("\r\n");
                                                sb.append("Content-Disposition: form-data;").append(" name=\"").append("username").append("\"");
                                                sb.append("\r\n\r\n").append(UserManager.getCurrentUser().getUsername()).append("\r\n");
                                                sb.append(splitLine).append("\r\n");
                                                sb.append("Content-Disposition: form-data;").append(" name=\"").append("password").append("\"");
                                                sb.append("\r\n\r\n").append(UserManager.getCurrentUser().getPassword()).append("\r\n");
                                                // 第一行
                                                // `"--" + BOUNDARY + "\r\n"`
                                                sb.append(splitLine).append("\r\n");
                                                // 第二行
                                                // Content-Disposition: form-data; name="参数的名称"; filename="上传的文件名" + "\r\n"
                                                sb.append("Content-Disposition: form-data;").append(" name=\"").append("avatar").append("\"");
                                                sb.append("; filename=\"").append("avatar.png").append("\"").append("\r\n");
                                                // 第三行
                                                // Content-Type: 文件的 mime 类型 + "\r\n"
                                                sb.append("Content-Type: ").append("image/png").append("\r\n");
                                                // 第四行
                                                // "\r\n"
                                                sb.append("\r\n");
                                                ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                                                try {
                                                    byteArrayOutputStream.write(sb.toString().getBytes("utf-8"));
                                                    // 第五行
                                                    // 文件的二进制数据 + "\r\n"
                                                    ByteArrayOutputStream b = new ByteArrayOutputStream();
                                                    BitmapUtil.createScaleBitmap(bmp, 640, 640)
                                                            .compress(Bitmap.CompressFormat.PNG, 80, b);
                                                    byte[] bytes = b.toByteArray();
                                                    byteArrayOutputStream.write(bytes);
                                                    byteArrayOutputStream.write("\r\n".getBytes("utf-8"));
                                                } catch (IOException e) {
                                                    e.printStackTrace();
                                                }
                                                // 结尾行
                                                // `"--" + BOUNDARY + "--" + "\r\n"`
                                                String endLine = splitLine + "--" + "\r\n";
                                                try {
                                                    byteArrayOutputStream.write(endLine.getBytes("utf-8"));
                                                } catch (IOException e) {
                                                    e.printStackTrace();
                                                }
                                                return byteArrayOutputStream.toByteArray();
                                            }

                                            @Override
                                            public String getBodyContentType() {
                                                return "multipart/form-data; boundary=----WebKitFormBoundaryYumin";
                                            }
                                        };
                                        avatarRequest.setShouldCache(false);
                                        // 设置请求的响应事件，因为文件上传需要较长的时间，所以在这里加大了，设为5秒
                                        avatarRequest.setRetryPolicy(new DefaultRetryPolicy(5000, DefaultRetryPolicy.DEFAULT_MAX_RETRIES, DefaultRetryPolicy.DEFAULT_BACKOFF_MULT));
                                        App.getRequestQueue().add(avatarRequest);
                                    }
                                })
                                .setNegativeButton(getString(R.string.operation_cancel), new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                    }
                                }).setCancelable(false).create().show();
                    });
                    break;
                case R.id.name:
                    Toast.makeText(this, "Name", Toast.LENGTH_SHORT).show();
                    break;
                case R.id.mail:
                    Toast.makeText(this, "Mail", Toast.LENGTH_SHORT).show();
                    break;
                case R.id.phone:
                    Toast.makeText(this, "Phone", Toast.LENGTH_SHORT).show();
                    break;
                case R.id.qq:
                    Toast.makeText(this, "QQ", Toast.LENGTH_SHORT).show();
                    break;
                case R.id.alertPassword:
                    break;
                case R.id.logout:
                    new AlertDialog.Builder(UserInfoActivity.this)
                            .setTitle("退出登录")
                            .setMessage("确定退出当前账户吗？")
                            .setPositiveButton(getString(R.string.operation_confirm), (dialog, which) -> {
                                UserManager.userLogout(UserManager.getCurrentUser());
                                user = null;
                                loadData();
                                Snackbar.make(getWindow().getDecorView(), "已退出当前用户", Snackbar.LENGTH_SHORT).show();
                                new Handler().postDelayed(() -> {
                                    LoginActivity.start(UserInfoActivity.this);
                                    // 发送销毁主活动的广播
                                    sendBroadcast(new Intent("cn.minbb.innovationabilityevaluationsystem.MainActivity.Finish"));
                                    UserInfoActivity.this.finish();
                                }, 2000);
                            })
                            .setNegativeButton(getString(R.string.operation_cancel), (dialog, which) -> {
                            }).create().show();
                    break;
                default:
            }
        } else {
            Snackbar.make(getWindow().getDecorView(), "您尚未登录", Snackbar.LENGTH_LONG).setAction("去登陆", v -> {
                LoginActivity.start(UserInfoActivity.this);
                UserInfoActivity.this.finish();
            }).show();
        }
    }
}
