package cn.minbb.evaluation.adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.view.ViewGroup;

import com.joanzapata.iconify.widget.IconTextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;

public class AnswerAdapter extends RecyclerView.Adapter<AnswerAdapter.ViewHolder> {

    // 映射数据
    private Context context;
    private List<ItemBean> itemBeen;

    private OnItemClickListener onItemClickListener;

    public AnswerAdapter(Context context, List<ItemBean> itemBeen) {
        this.context = context;
        this.itemBeen = itemBeen;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        // 转换一个 View 布局，决定了 item 的样子， 参数：1.上下文 2.Xml 布局资源 3.为 null
        View view = View.inflate(context, R.layout.waterfall_staggered_item, null);
        // 创建一个 StaggerViewHolder 对象并把 ViewHolder 对象传出去
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.nameText.setText(itemBeen.get(position).nameText);

        // 如果设置了回调，则设置点击事件
        if (onItemClickListener != null) {
            holder.itemView.findViewById(R.id.cardView).setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onItemClickListener.onItemClick(holder.itemView, holder.getLayoutPosition());
                }
            });

            holder.itemView.setOnLongClickListener(new View.OnLongClickListener() {
                @Override
                public boolean onLongClick(View v) {
                    onItemClickListener.onItemLongClick(holder.itemView, holder.getLayoutPosition());
                    return false;
                }
            });
        }
    }

    @Override
    public int getItemCount() {
        return itemBeen.size();
    }

    // ViewHolder 用于缓存控件
    class ViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.nameText)
        IconTextView nameText;

        ViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    public static class ItemBean {

        String nameText;

        public ItemBean(String nameText) {
            this.nameText = nameText;
        }
    }

    // 点击事件接口
    public interface OnItemClickListener {
        void onItemClick(View view, int position);

        void onItemLongClick(View view, int position);
    }

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.onItemClickListener = onItemClickListener;
    }
}
