package cn.minbb.evaluation.adapter;

import android.content.Context;
import android.support.v4.view.PagerAdapter;
import android.support.v7.widget.CardView;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

import cn.minbb.evaluation.R;
import cn.minbb.evaluation.entity.Answer;

public class CardPagerAdapter extends PagerAdapter implements CardAdapter {

    private Context context;
    private List<CardView> mViews;
    private List<CardItem> mData;
    private float mBaseElevation;

    private OnAnswerChangedListener onAnswerChangedListener;

    public CardPagerAdapter(Context context) {
        this.context = context;
        mViews = new ArrayList<>();
        mData = new ArrayList<>();
    }

    public void addCardItem(CardItem item) {
        mViews.add(null);
        mData.add(item);
    }

    public float getBaseElevation() {
        return mBaseElevation;
    }

    @Override
    public CardView getCardViewAt(int position) {
        return mViews.get(position);
    }

    @Override
    public int getCount() {
        return mData.size();
    }

    @Override
    public boolean isViewFromObject(View view, Object object) {
        return view == object;
    }

    @Override
    public Object instantiateItem(ViewGroup container, int position) {
        View view = LayoutInflater.from(container.getContext()).inflate(R.layout.question_card_item, container, false);
        container.addView(view);
        bind(mData.get(position), view);
        CardView cardView = view.findViewById(R.id.cardView);

        if (mBaseElevation == 0) {
            mBaseElevation = cardView.getCardElevation();
        }

        cardView.setMaxCardElevation(mBaseElevation * MAX_ELEVATION_FACTOR);
        mViews.set(position, cardView);
        return view;
    }

    @Override
    public void destroyItem(ViewGroup container, int position, Object object) {
        container.removeView((View) object);
        mViews.set(position, null);
    }

    private void bind(CardItem item, View view) {
        TextView questionTextView = view.findViewById(R.id.questionTextView);
        questionTextView.setText(item.getQuestion().getContent());
        RadioGroup answerGroupView = view.findViewById(R.id.answerGroupView);
        for (Answer answer : item.getQuestion().getAnswerSet()) {
            RadioButton radioButton = new RadioButton(context);
            radioButton.setTag(answer.getE_id());
            RadioGroup.LayoutParams layoutParams = new RadioGroup.LayoutParams(RadioGroup.LayoutParams.MATCH_PARENT, RadioGroup.LayoutParams.WRAP_CONTENT);
            layoutParams.setMargins(0, 0, 0, 0);
            radioButton.setLayoutParams(layoutParams);
            radioButton.setText(answer.getContent());
            radioButton.setTextSize(14);
            radioButton.setGravity(Gravity.CENTER_VERTICAL);
            radioButton.setPadding(0, 0, 0, 0);
            answerGroupView.addView(radioButton);
            radioButton.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    onAnswerChangedListener.onAnswerChanged(buttonView, isChecked, item);
                }
            });
        }
    }

    public interface OnAnswerChangedListener {
        void onAnswerChanged(CompoundButton buttonView, boolean isChecked, CardItem item);
    }

    public void setOnAnswerChangedListener(OnAnswerChangedListener onAnswerChangedListener) {
        this.onAnswerChangedListener = onAnswerChangedListener;
    }
}
