package cn.minbb.evaluation.adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.view.ViewGroup;

import com.joanzapata.iconify.widget.IconTextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;

public class PaperCardAdapter extends RecyclerView.Adapter<PaperCardAdapter.ViewHolder> {

    // 映射数据
    private Context context;
    private List<ItemBean> itemBeen;

    private OnItemClickListener onItemClickListener;
    private OnItemLongClickListener onItemLongClickListener;

    public PaperCardAdapter(Context context, List<ItemBean> itemBeen) {
        this.context = context;
        this.itemBeen = itemBeen;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        // 转换一个 View 布局，决定了 item 的样子， 参数：1.上下文 2.Xml 布局资源 3.为 null
        View view = View.inflate(context, R.layout.paper_card_item, null);
        // 创建一个 StaggerViewHolder 对象并把 ViewHolder 对象传出去
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.scoreText.setText(itemBeen.get(position).scoreText);
        holder.scoreLevel.setText(itemBeen.get(position).scoreLevel);

        // 如果设置了回调，则设置点击事件
        if (onItemClickListener != null) {
            holder.itemView.findViewById(R.id.cardView).setOnClickListener(v -> onItemClickListener.onItemClick(holder.itemView, holder.getLayoutPosition()));
        }

        if (onItemLongClickListener != null) {
            holder.itemView.findViewById(R.id.cardView).setOnLongClickListener(v -> {
                onItemLongClickListener.onItemLongClick(holder.itemView, holder.getLayoutPosition());
                return false;
            });
        }
    }

    @Override
    public int getItemCount() {
        return itemBeen.size();
    }

    // ViewHolder 用于缓存控件
    class ViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.scoreText)
        IconTextView scoreText;
        @BindView(R.id.scoreLevel)
        IconTextView scoreLevel;

        ViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    public static class ItemBean {

        String scoreText;
        String scoreLevel;

        public ItemBean(String scoreText, String scoreLevel) {
            this.scoreText = scoreText;
            this.scoreLevel = scoreLevel;
        }
    }

    // 点击事件接口
    public interface OnItemClickListener {
        void onItemClick(View view, int position);
    }

    // 长按事件接口
    public interface OnItemLongClickListener {
        void onItemLongClick(View view, int position);
    }

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.onItemClickListener = onItemClickListener;
    }

    public void setOnItemLongClickListener(OnItemLongClickListener onItemLongClickListener) {
        this.onItemLongClickListener = onItemLongClickListener;
    }
}
