package cn.minbb.evaluation.adapter;

import android.content.Context;
import android.support.v4.content.ContextCompat;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;

import com.joanzapata.iconify.widget.IconTextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;

public class PaperListAdapter extends BaseAdapter {

    // 映射数据
    private Context context;
    private List<ItemBean> itemBeen;
    private LayoutInflater layoutInflater;

    public PaperListAdapter(Context context, List<ItemBean> itemBeen) {
        this.context = context;
        layoutInflater = LayoutInflater.from(context);
        this.itemBeen = itemBeen;
    }

    @Override
    public int getCount() {
        // 获取数据量
        return itemBeen.size();
    }

    @Override
    public Object getItem(int position) {
        // 获取对应 ID 项对应的 Item
        return itemBeen.get(position);
    }

    @Override
    public long getItemId(int position) {
        // 获取对应项 ID
        return position;
    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
        ViewHolder viewHolder;
        if (convertView == null) {
            // 将 XML 转化为 View ，并不涉及到具体的布局，所以第二个参数通常设置为 null
            convertView = layoutInflater.inflate(R.layout.paper_list_item, parent, false);
            viewHolder = new ViewHolder(convertView);
            convertView.setTag(viewHolder);
        } else {
            viewHolder = (ViewHolder) convertView.getTag();
        }
        // 取出 ItemBean 对象
        ItemBean itemBean = itemBeen.get(position);
        // 设置控件数据
        Float score = itemBean.scoreText;
        Float mark = itemBean.markText;
        if (mark == null) {
            viewHolder.markText.setText("0.0");
            viewHolder.markLevel.setText("未完成");
            viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.gray));
        } else {
            float percent = mark / score * 100;
            viewHolder.markText.setText(String.format("%.1f", mark));
            if (percent >= 95) {
                viewHolder.markLevel.setText("A+");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_90_percent));
            } else if (percent >= 90) {
                viewHolder.markLevel.setText("A-");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_90_percent));
            } else if (percent >= 85) {
                viewHolder.markLevel.setText("B+");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_80_percent));
            } else if (percent >= 80) {
                viewHolder.markLevel.setText("B-");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_80_percent));
            } else if (percent >= 75) {
                viewHolder.markLevel.setText("C+");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_70_percent));
            } else if (percent >= 70) {
                viewHolder.markLevel.setText("C-");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_70_percent));
            } else if (percent >= 65) {
                viewHolder.markLevel.setText("D+");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_60_percent));
            } else if (percent >= 60) {
                viewHolder.markLevel.setText("D-");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_60_percent));
            } else if (percent >= 50) {
                viewHolder.markLevel.setText("E");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_50_percent));
            } else if (percent >= 40) {
                viewHolder.markLevel.setText("F");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_40_percent));
            } else if (percent >= 30) {
                viewHolder.markLevel.setText("G");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_30_percent));
            } else if (percent >= 20) {
                viewHolder.markLevel.setText("L");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_20_percent));
            } else if (percent >= 10) {
                viewHolder.markLevel.setText("XL");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_more_10_percent));
            } else if (percent < 10) {
                viewHolder.markLevel.setText("XXL");
                viewHolder.markLevel.setTextColor(ContextCompat.getColor(context, R.color.mark_less_10_percent));
            }
        }
        return convertView;
    }

    public static class ItemBean {

        private Float scoreText;
        private Float markText;

        public ItemBean(Float scoreText, Float markText) {
            this.scoreText = scoreText;
            this.markText = markText;
        }
    }

    static class ViewHolder {
        @BindView(R.id.markText)
        IconTextView markText;
        @BindView(R.id.markLevel)
        IconTextView markLevel;

        ViewHolder(View view) {
            ButterKnife.bind(this, view);
        }
    }
}
