package cn.minbb.evaluation.adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.joanzapata.iconify.widget.IconTextView;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.views.CircleImageView;
import cn.minbb.evaluation.ctrl.App;
import cn.minbb.evaluation.utils.OnDoubleClickListener;

public class PublicChatAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private List<MessageItemBean> messageItemBeanList;
    private LayoutInflater inflater;

    private OnItemClickListener onItemClickListener;
    private OnItemLongClickListener onItemLongClickListener;
    private OnItemDoubleClickListener onItemDoubleClickListener;

    public static final int SYSTEM = 0;
    public static final int RECEIVE = 1;
    public static final int SEND = 2;

    public PublicChatAdapter(Context context, List<MessageItemBean> messageItemBeanList) {
        this.messageItemBeanList = messageItemBeanList;
        this.inflater = LayoutInflater.from(context);
    }

    @Override
    public int getItemViewType(int position) {
        return messageItemBeanList.get(position).viewType;
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder viewHolder;
        switch (viewType) {
            case SYSTEM:
                viewHolder = new MessageSystemViewHolder(inflater.inflate(R.layout.public_chat_message_system_item, parent, false));
                break;
            case RECEIVE:
                viewHolder = new MessageReceiveViewHolder(inflater.inflate(R.layout.public_chat_message_received_item, parent, false));
                break;
            case SEND:
                viewHolder = new MessageSendViewHolder(inflater.inflate(R.layout.public_chat_message_send_item, parent, false));
                break;
            default:
                throw new IllegalArgumentException("视图类型错误！");
        }
        return viewHolder;
    }

    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder holder, int position) {
        MessageItemBean messageItemBean = messageItemBeanList.get(position);
        if (holder instanceof MessageSystemViewHolder) {
            ((MessageSystemViewHolder) holder).messageText.setText(messageItemBean.messageText);
        } else if (holder instanceof MessageReceiveViewHolder) {
            Glide.with(App.getApp()).load(messageItemBean.avatar).error(R.drawable.ic_action_person_dark).into(((MessageReceiveViewHolder) holder).avatar);
            ((MessageReceiveViewHolder) holder).sendTime.setText(messageItemBean.sendTime);
            ((MessageReceiveViewHolder) holder).nameText.setText(messageItemBean.nameText);
            ((MessageReceiveViewHolder) holder).messageText.setText(messageItemBean.messageText);

            if (onItemClickListener != null) {
                holder.itemView.findViewById(R.id.messageView).setOnClickListener(v -> onItemClickListener.onItemClick(holder.itemView, holder.getLayoutPosition()));
            }

            if (onItemLongClickListener != null) {
                holder.itemView.findViewById(R.id.messageView).setOnLongClickListener(v -> {
                    onItemLongClickListener.onItemLongClick(holder.itemView, holder.getLayoutPosition());
                    System.out.println("**********************************");
                    return false;
                });
            }

            if (onItemDoubleClickListener != null) {
                holder.itemView.findViewById(R.id.messageView).setOnTouchListener(new OnDoubleClickListener(new OnDoubleClickListener.DoubleClickCallback() {
                    @Override
                    public void onDoubleClick() {
                        onItemDoubleClickListener.onItemDoubleClick(holder.itemView, holder.getLayoutPosition());
                    }
                }));
            }
        } else if (holder instanceof MessageSendViewHolder) {
            Glide.with(App.getApp()).load(messageItemBean.avatar).error(R.drawable.ic_action_person_dark).into(((MessageSendViewHolder) holder).avatar);
            ((MessageSendViewHolder) holder).sendTime.setText(messageItemBean.sendTime);
            ((MessageSendViewHolder) holder).nameText.setText(messageItemBean.nameText);
            ((MessageSendViewHolder) holder).messageText.setText(messageItemBean.messageText);
            ((MessageSendViewHolder) holder).messageStatus.setText(messageItemBean.messageStatus);

            if (onItemClickListener != null) {
                holder.itemView.findViewById(R.id.messageView).setOnClickListener(v -> onItemClickListener.onItemClick(holder.itemView, holder.getLayoutPosition()));
            }

            if (onItemLongClickListener != null) {
                holder.itemView.findViewById(R.id.messageView).setOnLongClickListener(v -> {
                    onItemLongClickListener.onItemLongClick(holder.itemView, holder.getLayoutPosition());
                    return false;
                });
            }

            if (onItemDoubleClickListener != null) {
                holder.itemView.findViewById(R.id.messageView).setOnTouchListener(new OnDoubleClickListener(new OnDoubleClickListener.DoubleClickCallback() {
                    @Override
                    public void onDoubleClick() {
                        onItemDoubleClickListener.onItemDoubleClick(holder.itemView, holder.getLayoutPosition());
                    }
                }));
            }
        }
    }

    @Override
    public int getItemCount() {
        return messageItemBeanList.size();
    }

    class MessageSystemViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.messageText)
        IconTextView messageText;

        MessageSystemViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    class MessageReceiveViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.avatar)
        CircleImageView avatar;
        @BindView(R.id.sendTime)
        TextView sendTime;
        @BindView(R.id.nameText)
        IconTextView nameText;
        @BindView(R.id.messageText)
        IconTextView messageText;

        MessageReceiveViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    class MessageSendViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.avatar)
        CircleImageView avatar;
        @BindView(R.id.sendTime)
        TextView sendTime;
        @BindView(R.id.nameText)
        IconTextView nameText;
        @BindView(R.id.messageText)
        IconTextView messageText;
        @BindView(R.id.messageStatus)
        IconTextView messageStatus;

        MessageSendViewHolder(View view) {
            super(view);
            ButterKnife.bind(this, view);
        }
    }

    public static class MessageItemBean {

        int viewType;
        String avatar, sendTime, nameText, messageText, messageStatus;

        public MessageItemBean(int viewType, String avatar, String sendTime, String nameText, String messageText, String messageStatus) {
            this.viewType = viewType;
            this.avatar = avatar;
            this.sendTime = sendTime;
            this.nameText = nameText;
            this.messageText = messageText;
            this.messageStatus = messageStatus;
        }
    }

    // 单击事件接口
    public interface OnItemClickListener {
        void onItemClick(View view, int position);
    }

    // 长按事件接口
    public interface OnItemLongClickListener {
        void onItemLongClick(View view, int position);
    }

    // 双击事件接口
    public interface OnItemDoubleClickListener {
        void onItemDoubleClick(View view, int position);
    }

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.onItemClickListener = onItemClickListener;
    }

    public void setOnItemLongClickListener(OnItemLongClickListener onItemLongClickListener) {
        this.onItemLongClickListener = onItemLongClickListener;
    }

    public void setOnItemDoubleClickListener(OnItemDoubleClickListener onItemDoubleClickListener) {
        this.onItemDoubleClickListener = onItemDoubleClickListener;
    }
}
