package cn.minbb.evaluation.ctrl;

import android.annotation.SuppressLint;
import android.app.ActivityManager;
import android.app.Application;
import android.content.ComponentName;
import android.content.Context;
import android.os.Environment;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.AppCompatDelegate;
import android.support.v7.widget.Toolbar;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;

import com.activeandroid.ActiveAndroid;
import com.android.volley.RequestQueue;
import com.android.volley.toolbox.Volley;
import com.joanzapata.iconify.Iconify;
import com.joanzapata.iconify.fonts.EntypoModule;
import com.joanzapata.iconify.fonts.FontAwesomeModule;
import com.joanzapata.iconify.fonts.IoniconsModule;
import com.joanzapata.iconify.fonts.MaterialCommunityModule;
import com.joanzapata.iconify.fonts.MaterialModule;
import com.joanzapata.iconify.fonts.MeteoconsModule;
import com.joanzapata.iconify.fonts.SimpleLineIconsModule;
import com.joanzapata.iconify.fonts.TypiconsModule;
import com.joanzapata.iconify.fonts.WeathericonsModule;
import com.nostra13.universalimageloader.cache.disc.naming.Md5FileNameGenerator;
import com.nostra13.universalimageloader.core.ImageLoader;
import com.nostra13.universalimageloader.core.ImageLoaderConfiguration;
import com.nostra13.universalimageloader.core.assist.QueueProcessingType;

import org.wlf.filedownloader.FileDownloadConfiguration;
import org.wlf.filedownloader.FileDownloader;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import cn.minbb.evaluation.R;
import cn.minbb.evaluation.entity.User;

public class App extends Application {

    // 全局上下文
    private Context context = null;
    // Volley 网络请求队列
    private static RequestQueue requestQueue;
    // 时间格式
    private static SimpleDateFormat simpleDateFormat = (SimpleDateFormat) SimpleDateFormat.getDateTimeInstance();

    public App() {
    }

    @SuppressLint("StaticFieldLeak")
    private static App app = null;

    public static App getApp() {
        return app;
    }

    // 临时用户
    public static User user = null;

    @Override
    public void onCreate() {
        super.onCreate();
        app = this;
        context = getApplicationContext();

        // 初始化 ActiveAndroid ORM 框架 // 开启日志
        // ActiveAndroid.setLoggingEnabled(true);
        ActiveAndroid.initialize(this, true);
        // 初始化 Volley 网络请求队列
        requestQueue = Volley.newRequestQueue(context);
        // 初始化字体图标模块
        Iconify.with(new FontAwesomeModule())
                .with(new EntypoModule())
                .with(new TypiconsModule())
                .with(new MaterialModule())
                .with(new MaterialCommunityModule())
                .with(new MeteoconsModule())
                .with(new WeathericonsModule())
                .with(new SimpleLineIconsModule())
                .with(new IoniconsModule());

        File directory = new File(getAppDirectoryPath());
        if (!directory.exists()) {
            directory.mkdir();
        }
        // 初始化下载模块 file-downloader
        // 1.create FileDownloadConfiguration.Builder
        FileDownloadConfiguration.Builder builder = new FileDownloadConfiguration.Builder(this);
        // 2.config FileDownloadConfiguration.Builder
        builder.configFileDownloadDir(getAppDirectoryPath()); // config the download path
        // allow 3 download tasks at the same time, if not config, default is 2
        builder.configDownloadTaskSize(1);
        // config retry download times when failed, if not config, default is 0
        builder.configRetryDownloadTimes(3);
        // enable debug mode, if not config, default is false
        builder.configDebugMode(true);
        // config connect timeout, if not config, default is 15s
        builder.configConnectTimeout(25000); // 25s
        // 3.init FileDownloader with the configuration
        FileDownloadConfiguration configuration = builder.build(); // build FileDownloadConfiguration with the builder
        FileDownloader.init(configuration);

        // 初始化 ImageLoader
        if (!ImageLoader.getInstance().isInited()) {
            ImageLoaderConfiguration config = new ImageLoaderConfiguration.Builder(context)
                    .threadPriority(Thread.NORM_PRIORITY - 2)
                    .diskCacheSize(100 * 1024 * 1024)
                    .diskCacheFileNameGenerator(new Md5FileNameGenerator())
                    .tasksProcessingOrder(QueueProcessingType.LIFO)
                    .build();
            ImageLoader.getInstance().init(config);
        }
    }

    @Override
    public void onTerminate() {
        super.onTerminate();
        ActiveAndroid.dispose();
    }

    public static RequestQueue getRequestQueue() {
        return requestQueue;
    }

    // 获取系统当前时间
    public static String getCurrentStringTime() {
        return simpleDateFormat.format(getCurrentDateTime());
    }

    public static Date getCurrentDateTime() {
        return new Date();
    }

    public Context getContext() {
        return context;
    }

    // 获取 APP 文件夹路径
    public static String getAppDirectoryPath() {
        return Environment.getExternalStorageDirectory().getAbsolutePath() + File.separator + "Evaluation";
    }

    // 设置 ActionBar
    public static void setupActionBar(AppCompatActivity appCompatActivity) {
        // id from appcompat
        ViewGroup rootView = appCompatActivity.findViewById(R.id.action_bar_root);
        if (rootView != null) {
            View view = appCompatActivity.getLayoutInflater().inflate(R.layout.app_bar_layout, rootView, false);
            rootView.addView(view, 0);

            Toolbar toolbar = appCompatActivity.findViewById(R.id.toolbar);
            appCompatActivity.setSupportActionBar(toolbar);
        }

        ActionBar actionBar = appCompatActivity.getSupportActionBar();
        if (actionBar != null) {
            // 在 ActionBar 中显示 UP 按钮。
            actionBar.setDisplayHomeAsUpEnabled(true);
        }
    }

    public static boolean activityIsForeground(Context context, String className) {
        if (context == null || TextUtils.isEmpty(className)) {
            return false;
        }
        ActivityManager activityManager = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
        List<ActivityManager.RunningTaskInfo> list = activityManager.getRunningTasks(1);
        if (list != null && list.size() > 0) {
            ComponentName componentName = list.get(0).topActivity;
            return className.equals(componentName.getClassName());
        }
        return false;
    }
}
