package cn.minbb.evaluation.ctrl;

import android.Manifest;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;

import java.util.ArrayList;
import java.util.Iterator;

import cn.minbb.evaluation.utils.Constant;

// 动态申请权限
public class PermissionManager {

    private Activity activity;

    // 定义一个list，用于存储需要申请的权限
    private ArrayList<String> permissionList = new ArrayList<>();

    public PermissionManager(Activity activity) {
        this.activity = activity;
    }

    public PermissionManager addPermission(String permission) {
        permissionList.add(Manifest.permission.CAMERA);
        permissionList.add(Manifest.permission.RECORD_AUDIO);
        permissionList.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
        return this;
    }

    // 调用封装好的申请权限的方法
    public void checkAndRequestPermissions() {
        ArrayList<String> list = new ArrayList<>(permissionList);
        Iterator<String> iterator = list.iterator();
        while (iterator.hasNext()) {
            String permission = iterator.next();
            // 检查权限是否已经申请
            int hasPermission = ContextCompat.checkSelfPermission(activity, permission);
            if (hasPermission == PackageManager.PERMISSION_GRANTED) {
                iterator.remove();
            }
        }
        /**
         补充说明：
         ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.RECORD_AUDIO);
         对于原生Android，如果用户选择了“不再提示”，那么shouldShowRequestPermissionRationale就会为true。
         此时，用户可以弹出一个对话框，向用户解释为什么需要这项权限。
         对于一些深度定制的系统，如果用户选择了“不再提示”，那么shouldShowRequestPermissionRationale永远为false
         */
        if (list.size() != 0) {
            String[] permissions = list.toArray(new String[0]);
            // 正式请求权限
            ActivityCompat.requestPermissions(activity, permissions, Constant.PERMISSIONS_REQUEST);
        }
    }

    public void checkAndRequestPermissions(ArrayList<String> permissionList) {
        ArrayList<String> list = new ArrayList<>(permissionList);
        Iterator<String> iterator = list.iterator();
        while (iterator.hasNext()) {
            String permission = iterator.next();
            // 检查权限是否已经申请
            int hasPermission = ContextCompat.checkSelfPermission(activity, permission);
            if (hasPermission == PackageManager.PERMISSION_GRANTED) {
                iterator.remove();
            }
        }

        if (list.size() != 0) {
            String[] permissions = list.toArray(new String[0]);
            // 正式请求权限
            ActivityCompat.requestPermissions(activity, permissions, Constant.PERMISSIONS_REQUEST);
        }
    }
}
