package cn.minbb.evaluation.ctrl;

import android.content.Context;
import android.content.SharedPreferences;

import cn.minbb.evaluation.entity.Version;

public class VersionManager {

    private static final String PREFERENCE_VERSION = "version";

    /**
     * 将版本信息写入本地文件保存
     *
     * @param version 版本信息实体
     */
    public static void setVersion(Version version) {
        SharedPreferences.Editor editorVersion = App.getApp().getSharedPreferences(PREFERENCE_VERSION, Context.MODE_PRIVATE).edit();
        editorVersion.putInt("version_code", version.getVersionCode());
        editorVersion.putString("version_name", version.getVersionName());
        editorVersion.putString("version_content", version.getContent());
        editorVersion.putInt("file_size", version.getFileSize());
        editorVersion.putString("file_url", version.getFileUrl());
        editorVersion.putBoolean("forced_update", version.getForced());
        editorVersion.apply();
    }

    /**
     * 从本地文件中获取版本信息
     *
     * @return 版本信息实体
     */
    public static Version getVersion() {
        SharedPreferences sharedPreferencesVersion = App.getApp().getSharedPreferences(PREFERENCE_VERSION, Context.MODE_PRIVATE);
        Version version = new Version();
        version.setVersionCode(sharedPreferencesVersion.getInt("version_code", 0));
        version.setVersionName(sharedPreferencesVersion.getString("version_name", ""));
        version.setContent(sharedPreferencesVersion.getString("version_content", ""));
        version.setFileSize(sharedPreferencesVersion.getInt("file_size", 0));
        version.setFileUrl(sharedPreferencesVersion.getString("file_url", ""));
        version.setForced(sharedPreferencesVersion.getBoolean("forced_update", true));
        return version;
    }

    /**
     * 将要忽略更新的版本号写入本地文件
     *
     * @param code 忽略更新的版本号
     */
    public static void setIgnoreVersionCode(int code) {
        SharedPreferences.Editor editorIgnoreCode = App.getApp().getSharedPreferences(PREFERENCE_VERSION, Context.MODE_PRIVATE).edit();
        editorIgnoreCode.putInt("ignore_version_code", code);
        editorIgnoreCode.apply();
    }

    /**
     * 获取要忽略更新的版本号
     *
     * @return 忽略更新的版本号
     */
    public static int getIgnoreVersionCode() {
        return App.getApp().getSharedPreferences(PREFERENCE_VERSION, Context.MODE_PRIVATE).getInt("ignore_version_code", 0);
    }
}
