package cn.minbb.evaluation.dao;

import com.activeandroid.ActiveAndroid;
import com.activeandroid.query.Delete;
import com.activeandroid.query.Select;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import cn.minbb.evaluation.entity.Paper;

public class PaperDao extends Dao<Paper> {

    private static final PaperDao PAPER_DAO = new PaperDao();

    private PaperDao() {
    }

    public static PaperDao getPaperDao() {
        return PAPER_DAO;
    }

    @Override
    public Paper findById(Long id) {
        return new Select().from(Paper.class).where("id = ?", id).executeSingle();
    }

    @Override
    public Paper findByE_Id(Long e_id) {
        return new Select().from(Paper.class).where("e_id = ?", e_id).executeSingle();
    }

    @Override
    public List<Paper> findAll() {
        return new Select().from(Paper.class).execute();
    }

    @Override
    public Paper save(Paper var) {
        Paper paper = null;
        long id = var.save();
        if (id != -1) {
            paper = findById(id);
        }
        return paper;
    }

    @Override
    public List<Paper> saveAll(Collection<Paper> var) {
        List<Paper> paperList = new ArrayList<>(var.size());
        ActiveAndroid.beginTransaction();
        try {
            for (Paper paper : var) {
                long id = paper.save();
                if (id != -1) {
                    paperList.add(findById(id));
                }
            }
            ActiveAndroid.setTransactionSuccessful();
        } finally {
            ActiveAndroid.endTransaction();
        }
        return paperList;
    }

    @Override
    public void delete(Paper var) {
        var.delete();
    }

    @Override
    public void deleteAll() {
        new Delete().from(Paper.class).execute();
    }
}
