package cn.minbb.evaluation.fragment;

import android.content.Context;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.bumptech.glide.Glide;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import cn.minbb.evaluation.R;
import cn.minbb.evaluation.ctrl.UserManager;
import uk.co.senab.photoview.PhotoView;
import uk.co.senab.photoview.PhotoViewAttacher;

public class AvatarFragment extends Fragment implements PhotoViewAttacher.OnPhotoTapListener, View.OnKeyListener {

    @BindView(R.id.avatar)
    PhotoView avatar;

    private Unbinder unbinder;

    private PhotoViewAttacher photoViewAttacher;

    public AvatarFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_avatar, container, false);
        unbinder = ButterKnife.bind(this, view);

        Glide.with(this)
                .load(UserManager.getCurrentUser().getAvatar())
                .placeholder(R.mipmap.ic_launcher)
                .error(R.mipmap.ic_launcher)
                .into(avatar);
        photoViewAttacher = new PhotoViewAttacher(avatar);
        photoViewAttacher.setOnPhotoTapListener(this);
        return view;
    }

    @Override
    public void onResume() {
        super.onResume();
        View view = getView();
        if (view != null) {
            view.setFocusable(true);
            view.setFocusableInTouchMode(true);
            view.requestFocus();
            view.setOnKeyListener(this);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        photoViewAttacher.setOnPhotoTapListener(null);
        photoViewAttacher = null;
        System.gc();
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
    }

    @Override
    public void onDetach() {
        super.onDetach();
    }

    @Override
    public void onPhotoTap(View view, float v, float v1) {
        getFragmentManager().popBackStack();
    }

    @Override
    public boolean onKey(View v, int keyCode, KeyEvent event) {
        if (event.getAction() == KeyEvent.ACTION_DOWN && keyCode == KeyEvent.KEYCODE_BACK) {
            // 监听到返回按钮点击事件后销毁此 Fragment
            getFragmentManager().popBackStack();
            // 未处理
            return true;
        }
        return false;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        unbinder.unbind();
    }
}
