package cn.minbb.evaluation.views;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.AnimationSet;
import android.view.animation.DecelerateInterpolator;
import android.view.animation.ScaleAnimation;
import android.view.animation.TranslateAnimation;
import android.widget.RelativeLayout;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.Set;

import cn.minbb.evaluation.R;

public class ClickShowTextLayout extends RelativeLayout {

    private List<String> stringList;

    public ClickShowTextLayout(Context context) {
        super(context);
        clickShow();
    }

    public ClickShowTextLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        clickShow();
    }

    public ClickShowTextLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        clickShow();
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                break;
            case MotionEvent.ACTION_MOVE:
                break;
            case MotionEvent.ACTION_UP:
                Random random = new Random();
                TextView textView = new TextView(getContext());
                textView.setText(stringList.get(random.nextInt(stringList.size())));
                textView.setTextColor(true ? Color.rgb(random.nextInt(256), random.nextInt(256), random.nextInt(256)) : getResources().getColor(R.color.colorAccent));
                // 测量 View 宽高
                textView.measure(View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED), View.MeasureSpec.makeMeasureSpec(0, View.MeasureSpec.UNSPECIFIED));
                int height = textView.getMeasuredHeight();
                int width = textView.getMeasuredWidth();
                // 动画集
                AnimationHelper animationHelper = new AnimationHelper();
                AnimationSet animationSet = new AnimationSet(false);
                animationSet.addAnimation(animationHelper.createTranslateAnim());
                animationSet.addAnimation(animationHelper.createAlphaAnim());
                animationSet.addAnimation(animationHelper.createScaleAnim());
                animationSet.setAnimationListener(new Animation.AnimationListener() {
                    @Override
                    public void onAnimationStart(Animation animation) {
                    }

                    @Override
                    public void onAnimationEnd(Animation animation) {
                        // 在线程中移除视图
                        new Handler().post(() -> removeView(textView));
                    }

                    @Override
                    public void onAnimationRepeat(Animation animation) {
                    }
                });
                textView.startAnimation(animationSet);
                // 位置样式
                RelativeLayout.LayoutParams params = new LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
                params.setMargins((int) event.getX() - width / 2, (int) event.getY() - height / 2, 0, 0);
                textView.setLayoutParams(params);
                addView(textView);
                break;
            default:
        }
        /**
         * 返回值
         * true：view 继续响应 Touch 操作；
         * false：view不再响应 Touch 操作，故此处若为 false，只能显示起始位置，不能显示实时位置和结束位置
         */
        return true;
    }

    private void clickShow() {
        String[] defaults = new String[]{"程序员", "不爱Bug", "安静", "自由", "爱生活", "技术宅", "科技男", "双鱼座", "完美主义", "乐观", "强迫症晚期", "爱涵涵儿"};
        stringList = Arrays.asList(defaults);
    }

    public void clickShow(String[] strings) {
        stringList = Arrays.asList(strings);
    }

    public void clickShow(List<String> strings) {
        stringList = strings;
    }

    public void clickShow(Set<String> strings) {
        stringList = new ArrayList<>(strings);
    }

    private class AnimationHelper {

        // 平移动画
        public Animation createTranslateAnim() {
            TranslateAnimation translateAnimation = new TranslateAnimation(
                    Animation.RELATIVE_TO_SELF, 0,
                    Animation.RELATIVE_TO_SELF, 0,
                    Animation.RELATIVE_TO_SELF, 0,
                    Animation.RELATIVE_TO_SELF, -1);
            translateAnimation.setDuration(1500);
            translateAnimation.setInterpolator(new DecelerateInterpolator());
            translateAnimation.setFillAfter(true);
            return translateAnimation;
        }

        // 透明度动画
        public Animation createAlphaAnim() {
            AlphaAnimation alphaAnimation = new AlphaAnimation(1.0f, 0.1f);
            alphaAnimation.setDuration(1500);
            alphaAnimation.setFillAfter(true);
            return alphaAnimation;
        }

        // 缩放动画
        public Animation createScaleAnim() {
            ScaleAnimation scaleAnimation = new ScaleAnimation(
                    1, 0.1f,
                    1, 0.1f,
                    Animation.RELATIVE_TO_SELF, 0.5f,
                    Animation.RELATIVE_TO_SELF, -1);
            scaleAnimation.setDuration(1500);
            scaleAnimation.setFillAfter(true);
            return scaleAnimation;
        }
    }
}
