package cn.minbb.evaluation.views;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AbsListView;
import android.widget.ListView;

import cn.minbb.evaluation.R;

public class PullUpLoadMoreListView extends ListView implements AbsListView.OnScrollListener {

    private boolean isLoading = false;
    private View footerView;
    private int footerHeight;
    private OnPullUpLoadMoreListener listener;
    private LayoutInflater inflater;

    public PullUpLoadMoreListView(Context context) {
        super(context);
    }

    public PullUpLoadMoreListView(Context context, AttributeSet attrs) {
        super(context, attrs);

        inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        initFooterView();
        setOnScrollListener(this);
    }

    public PullUpLoadMoreListView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    /**
     * 初始化布局
     */
    private void initFooterView() {
        footerView = inflater.inflate(R.layout.refresh_load_more, null);
        footerView.measure(0, 0);
        footerHeight = footerView.getMeasuredHeight();
        footerView.setPadding(0, -footerHeight, 0, 0);
        this.addFooterView(footerView, null, false);
    }

    @Override
    public void onScrollStateChanged(AbsListView absListView, int scrollState) {
        if (this.getLastVisiblePosition() == this.getAdapter().getCount() - 1
                && !isLoading && (scrollState == SCROLL_STATE_FLING || scrollState == SCROLL_STATE_IDLE)) {
            setLoadState(true);
            if (this.listener != null) {
                this.listener.onLoadMore(this);
            }
        }
    }

    /**
     * 设置状态
     *
     * @param b
     */
    public void setLoadState(boolean b) {
        this.isLoading = b;
        if (isLoading) {
            footerView.setPadding(0, 0, 0, 0);
            this.setSelection(this.getAdapter().getCount() + 1);
        } else {
            footerView.setPadding(0, -footerHeight, 0, 0);
        }
    }

    public boolean isLoading() {
        return this.isLoading;
    }

    public void loadComplete() {
        setLoadState(false);
    }

    @Override
    public void onScroll(AbsListView absListView, int i, int i1, int i2) {

    }

    public void setOnLoadMoreListener(OnPullUpLoadMoreListener listener) {
        this.listener = listener;
    }

    public interface OnPullUpLoadMoreListener {
        void onLoadMore(PullUpLoadMoreListView pullUpLoadMoreListView);
    }
}
